/*
 *	EXAM10.C
 *
 *	Simple gate-level logic simulator prototype application using
 *	virtual channels and the FORK/JOIN concurrency primitives as described
 *	in the "EXAMPLES" section of the Logical Systems "TRANSPUTER 'C'
 *	LIBRARY DESCRIPTION" documentation.
 *
 *	The "and.nif", "or.nif" and "nand.nif" files contain suitable network
 *	information files for running the program.
 */

#include	<stdio.h>
#include	<stdlib.h>
#include	<conc.h>

#define		WS_SIZE		1024	/* Workspace size/process */

void	*
get_ws(void)
	{				/* Get a process workspace */
	void			*rval;

	if((rval = malloc((size_t) WS_SIZE)) == NULL)
		{
		printf("Unable to 'malloc' storage for process workspace\n");
		exit(1);
		}
	return (rval);
	}

void
nand(int input1,int input2,int output)
	{				/* Perform logical NAND operation */
	Channel			*in1;
	Channel			*in2;
	Channel			*out;

	in1 = VChan(input1);		/* Get channel addresses */
	in2 = VChan(input2);
	out = VChan(output);

	if((in1 == NULL) || (in2 == NULL) || (out == NULL))
		PStop();		/* Die if not needed */

	while(1)
		VChanOutChar(out,! (VChanInChar(in1) & VChanInChar(in2)));
	}

void
wired_or(int input,int output1,int output2)
	{				/* Duplicate input on two outputs */
	int			data;
	Forkblk			f;
	Channel			*in;
	Channel			*out1;
	Channel			*out2;
	PDes			p;
	void			*ws;

	in = VChan(input);		/* Get channel addresses */
	out1 = VChan(output1);
	out2 = VChan(output2);

	if((in == NULL) || (out1 == NULL) || (out2 == NULL))
		PStop();		/* Die if not needed */

	ws = get_ws();			/* Get workspace for parallel output */

	while(1)
		{
		data = VChanInChar(in);
		PForkInit(f,2);		/* Outputs to be done in parallel */
		p = PSetup(ws,VChanOutChar,WS_SIZE,2,out1,data);
		PFork(f,p);
		VChanOutChar(out2,data);
		PJoin(&f);		/* Synchronize processes */
		}
	}

void
main()
	{
	Forkblk			f;
	int			i;
	Channel			*in;
	Channel			*out1;
	Channel			*out2;
	PDes			p;
	void			*ws;
/*
 *	Convert logical virtual channel numbers to channel pointers.
 */
	in = VChan(6);
	out1 = VChan(7);
	out2 = VChan(8);

	if((in == NULL) || (out1 == NULL) || (out2 == NULL))
		{
		printf("Virtual channels not correctly configured\n");
		exit(1);
		}
/*
 *	Run three NAND processes.
 */
	PRun(PSetup(get_ws(),nand,WS_SIZE,3,9,10,11) | 1);
	PRun(PSetup(get_ws(),nand,WS_SIZE,3,12,13,14) | 1);
	PRun(PSetup(get_ws(),nand,WS_SIZE,3,15,16,17) | 1);
/*
 *	Run two WIRED-OR processes.
 */
	PRun(PSetup(get_ws(),wired_or,WS_SIZE,3,18,19,20) | 1);
	PRun(PSetup(get_ws(),wired_or,WS_SIZE,3,21,22,23) | 1);
/*
 *	Stimulate "external" logic function and display results.
 */
	ws = get_ws();			/* Get workspace for parallel output */

	printf("Truth Table For Externally Configured Function\n\n");
	printf("      Input1  Input2    |     Output\n");
	printf("      ------------------------------\n");
	for(i = 0; i < 4; i++)
		{
		printf("\t%d\t%d\t|\t",(i & 1) != 0,(i & 2) != 0);
		PForkInit(f,2);		/* Outputs to be done in parallel */
		p = PSetup(ws,VChanOutChar,WS_SIZE,2,out1,((i & 1) != 0));
		PFork(f,p);
		VChanOutChar(out2,((i & 2) != 0));
		PJoin(&f);		/* Synchronize processes */
		printf("%d\n",VChanInChar(in));
		}
	}
