/*
 *	EXAM6.C
 *
 *	Four Transputer DSP "filter" program as described in the "EXAMPLES"
 *	section of the Logical Systems "TRANSPUTER 'C' LIBRARY DESCRIPTION"
 *	documentation.  The "pipe4.nif" file contains a suitable topology file
 *	for use with LD-NET when running this program.
 *
 *	WARNING:  This program uses floating point and should only be used with
 *	32 bit Transputers!
 */

#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>
#include	<conc.h>

#define		DATA_SETS	100	/* # of data sets to do */
#define		NUM_POINTS	32	/* # of complex data points */
#define		LOG_POINTS	5	/* Log base 2 of NUM_POINTS */
#define		WS_SIZE		8192	/* Workspace size/process */

COMPLEX		in_data[NUM_POINTS];
COMPLEX		out_data[NUM_POINTS];

void
inbuf(Process *p)			/* Write data to pipeline */
	{
	int			i,j;

	for(i = 0; i < DATA_SETS; i++)
		{
		for(j = 0; j < NUM_POINTS; j++)	/* Random */
			{
			out_data[j].real = frand();
			out_data[j].imag = frand();
			}
		ChanOut(LINK1OUT,out_data,(int) sizeof(out_data));
		}
	}

void
outbuf(Process *p)			/* Read data from pipeline */
	{
	int			i;

	for(i = 0; i < DATA_SETS; i++)	/* Read and toss */
		ChanIn(LINK2IN,in_data,(int) sizeof(in_data));
	}

main()
	{
	if(_node_number == 1)		/* Root node */
		{
		Process		*in,*out;

		if((in = ProcAlloc(inbuf,WS_SIZE,0)) == NULL)
			{
			printf("No memory for process 'inbuf'\n");
			exit(1);
			}
		if((out = ProcAlloc(outbuf,WS_SIZE,0)) == NULL)
			{
			printf("No memory for process 'outbuf'\n");
			exit(1);
			}
		ProcPar(in,out,NULL);	/* Start I/O processes */
		ProcFree(in);		/* Return heap storage */
		ProcFree(out);
		printf("All done\n");
		}
	else if(_node_number == 2)	/* 1st pipeline stage */
		{
		while(1)
			{
			ChanIn(LINK2IN,in_data,(int) sizeof(in_data));
			fft(in_data,LOG_POINTS);
			ChanOut(LINK1OUT,in_data,(int) sizeof(in_data));
			}
		}
	else if(_node_number == 3)	/* 2nd pipeline stage */
		{
		int		i;

		while(1)
			{
			ChanIn(LINK2IN,in_data,(int) sizeof(in_data));
/* DC term */		in_data[0].real = in_data[0].imag = 0.0;
			for(i = 0; i < NUM_POINTS; i++)
				{
/* Normalize */			in_data[i].real /= NUM_POINTS;
				in_data[i].imag /= NUM_POINTS;
				}
			ChanOut(LINK1OUT,in_data,(int) sizeof(in_data));
			}
		}
	else				/* 3rd pipeline stage */
		{
		while(1)
			{
			ChanIn(LINK2IN,in_data,(int) sizeof(in_data));
			ifft(in_data,LOG_POINTS);
			ChanOut(LINK1OUT,in_data,(int) sizeof(in_data));
			}
		}
	}
