
/* Graphics functions */
/* CopyRight (c) CSA 1990 */

enum graphics_errors {          /* graphresult error return codes */
	grOk               =   0,
	grNoInitGraph      =  -1,
	grNotDetected      =  -2,
	grFileNotFound     =  -3,
	grInvalidDriver    =  -4,
	grNoLoadMem        =  -5,
	grNoScanMem        =  -6,
	grNoFloodMem       =  -7,
	grFontNotFound     =  -8,
	grNoFontMem        =  -9,
	grInvalidMode      = -10,
	grError            = -11,   /* generic error */
	grIOerror          = -12,
	grInvalidFont      = -13,
	grInvalidFontNum   = -14,
	grInvalidVersion   = -18
};

enum graphics_drivers {         /* define graphics drivers */
	DETECT,                 /* requests autodetection */
	EGA=3, VGA = 9, CURRENT_DRIVER = -1
};

enum graphics_modes {           /* graphics modes for each driver */
    EGALO      = 0,  /* 640x200 16 color 4 pages        */
    EGAHI      = 1,  /* 640x350 16 color 2 pages        */
    VGALO      = 0,  /* 640x200 16 color 4 pages        */
    VGAMED     = 1,  /* 640x350 16 color 2 pages        */
    VGAHI      = 2  /* 640x480 16 color 1 page         */
};

/* Colors for setpalette and setallpalette */

#if     !defined(__COLORS)
#define __COLORS

enum COLORS {
    BLACK,                  /* dark colors */
    BLUE,
    GREEN,
    CYAN,
    RED,
    MAGENTA,
    BROWN,
    LIGHTGRAY,
    DARKGRAY,               /* light colors */
    LIGHTBLUE,
    LIGHTGREEN,
    LIGHTCYAN,
    LIGHTRED,
    LIGHTMAGENTA,
    YELLOW,
    WHITE
};
#endif

enum EGA_COLORS {
    EGA_BLACK            =  0,      /* dark colors */
    EGA_BLUE             =  1,
    EGA_GREEN            =  2,
    EGA_CYAN             =  3,
    EGA_RED              =  4,
    EGA_MAGENTA          =  5,
    EGA_BROWN            =  20,
    EGA_LIGHTGRAY        =  7,
    EGA_DARKGRAY         =  56,     /* light colors */
    EGA_LIGHTBLUE        =  57,
    EGA_LIGHTGREEN       =  58,
    EGA_LIGHTCYAN        =  59,
    EGA_LIGHTRED         =  60,
    EGA_LIGHTMAGENTA     =  61,
    EGA_YELLOW           =  62,
    EGA_WHITE            =  63
};

enum line_styles {              /* Line styles for get/setlinestyle */
	SOLID_LINE   = 0,
	DOTTED_LINE  = 1,
	CENTER_LINE  = 2,
	DASHED_LINE  = 3,
	USERBIT_LINE = 4        /* User defined line style */
};

enum line_widths {              /* Line widths for get/setlinestyle */
	NORM_WIDTH  = 1,
	THICK_WIDTH = 3
};

enum font_names {
	DEFAULT_FONT    = 0,    /* 8x8 bit mapped font */
	TRIPLEX_FONT    = 1,    /* "Stroked" fonts */
	SMALL_FONT      = 2,
	SANS_SERIF_FONT = 3,
	GOTHIC_FONT     = 4
};

#define HORIZ_DIR       0       /* left to right */
#define VERT_DIR        1       /* bottom to top */

#define USER_CHAR_SIZE  0       /* user-defined char size */

enum fill_patterns {            /* Fill patterns for get/setfillstyle */
	EMPTY_FILL,             /* fills area in background color */
	SOLID_FILL,             /* fills area in solid fill color */
	LINE_FILL,              /* --- fill */
	LTSLASH_FILL,           /* /// fill */
	SLASH_FILL,             /* /// fill with thick lines */
	BKSLASH_FILL,           /* \\\ fill with thick lines */
	LTBKSLASH_FILL,         /* \\\ fill */
	HATCH_FILL,             /* light hatch fill */
	XHATCH_FILL,            /* heavy cross hatch fill */
	INTERLEAVE_FILL,        /* interleaving line fill */
	WIDE_DOT_FILL,          /* Widely spaced dot fill */
	CLOSE_DOT_FILL,         /* Closely spaced dot fill */
	USER_FILL               /* user defined fill */
};

enum putimage_ops {             /* BitBlt operators for putimage */
	COPY_PUT,               /* MOV */
	XOR_PUT,                /* XOR */
	OR_PUT,                 /* OR  */
	AND_PUT,                /* AND */
	NOT_PUT                 /* NOT */
};

enum text_just {                /* Horizontal and vertical justification
				   for settextjustify */
	LEFT_TEXT       = 0,
	CENTER_TEXT     = 1,
	RIGHT_TEXT      = 2,

	BOTTOM_TEXT     = 0,
     /* CENTER_TEXT     = 1,  already defined above */
	TOP_TEXT        = 2
};


#define MAXCOLORS 15

struct palettetype {
	unsigned char size;
	signed char colors[MAXCOLORS+1];
};

struct linesettingstype {
	int linestyle;
	unsigned upattern;
	int thickness;
};

struct textsettingstype {
	int font;
	int direction;
	int charsize;
	int horiz;
	int vert;
};

struct fillsettingstype {
	int pattern;
	int color;
};

struct pointtype {
	int x, y;
};

struct viewporttype {
	int left, top, right, bottom;
	int clip;
};

struct arccoordstype {
	int x, y;
	int xstart, ystart, xend, yend;
};

void arc(int x, int y,int startangle,int endangle,int radius);
void bar(int left,int top,int right,int bottom);
void bar3d(int left,int top,int right,int bottom,int depth,int topflag);
void circle(int x,int y,int radius);
void cleardevice();
void clearviewport();
void closegraph();
void detectgraph(int *graphdriver,int *graphmode);
void drawpoly(int numpoints, int *polypoints);
void ellipse(int x, int y, int stangle, int endangle, int xradius, int yradius);
void fillellipse(int x, int y, int xradius, int yradius);
void fillpoly(int numpoints, int *polypoints);
void floodfill(int x, int y, int border);
void getarccoords(struct arccoordstype *arccoords);
void getaspectratio(int *xasp, int *yasp);
int  getbkcolor();
int  getcolor();
void getdefaultpalette(struct palettetype *dp);
void getdrivername(char *name);
void getfillpattern(char *pattern);
void getfillsettings(struct fillsettingstype *fillinfo);
int  getgraphmode();
void getimage(int left,int top,int right,int bottom,unsigned *pointer);
void getlinesettings(struct linesettingstype *lineinfo);
int  getmaxcolor();
int  getmaxmode();
int  getmaxx();
int  getmaxy();
void getmodename(int num,char *name);
void getmoderange(int graphdriver,int *lomode, int *himode);
void getpalette(struct palettetype *palette);
int  getpalettesize();
unsigned getpixel(int x, int y);
void gettextsettings(struct textsettingstype *texttypeinfo);
void getviewsettings(struct viewporttype *viewport);
int  getx();
int  gety();
void graphdefaults();
void grapherrormsg(int errorcode,char *buf);
int  graphresult();
unsigned imagesize(int left,int top,int right,int bottom);
void initgraph(int *gdriver, int *gmode,char *path);
void line(int x1,int y1,int x2, int y2);
void linerel(int x1,int y1);
void lineto(int x1,int y1);
void moverel(int x1,int y1);
void moveto(int x1,int y1);
void outtext(char *textstring);
void outtextxy(int x, int y,char *textstring);
void pieslice(int x, int y, int stangle, int endangle, int radius);
void putimage(int left, int top, unsigned bitmap, int op);
void putpixel(int x, int y, int color);
void rectangle(int left, int top, int right, int bottom);
void restorecrtmode();
void sector(int x, int y, int stangle, int endangle, int xradius, int yradius);
void setactivepage(int page);
void setallpalette(struct palettetype *palette);
void setaspectratio(int xasp, int yasp);
void setbkcolor(int color);
void setcolor(int color);
void setfillpattern(char *pattern, int color);
void setfillstyle(int pattern, int color);
int setgraphbufsize(int bufsize);
void setgraphmode(int mode);
void setlinestyle(int linestyle, unsigned pattern, int thickness);
void setpalette(int colornum, int color);
void settextjustify(int hori, int vert);
void settextstyle(int font, int direction, int charsize);
void setusercharsize(int multx, int divx, int multy, int divy);
void setviewport(int left, int top ,int right, int bottom, int clip);
void setvisualpage(int page);
void setwritemode(int mode);
int  textheight(char *str);
int  textwidth(char *str);
void freeimage(unsigned pointer);

