/*
 *	cio.h	"C" I/O communication server definitions
 *
 *	Copyright 1987-1992 by Logical Systems.
 */

#define	MAX_MSG_DATA	25000		/* Max length of msg data contents */
#define	MAX_MSG_OVER	50		/* Max length of msg overhead */

/*
 *	SEEK origin cardinal values for cio/tcio.
 */

#define	_SEEK_SET	0
#define	_SEEK_CUR	1
#define	_SEEK_END	2

/*
 *	OPEN "access" cardinal values for cio/tcio.
 */

#define	_O_APPEND	0x0001		/* All write append */
#define	_O_BINARY	0x0002		/* No text translation (binary file) */
#define	_O_CREAT 	0x0004		/* If file doesn't exist create it */
#define	_O_EXCL		0x0008		/* Error if O_CREAT and file exists */
#define	_O_NONBLOCK	0x0010		/* Return if would block */
#define	_O_RDONLY	0x0020		/* Open file for reading */
#define	_O_RDWR		0x0040		/* Open file for reading & writing */
#define	_O_TEXT		0x0080		/* Perform translation (text file) */
#define	_O_TRUNC 	0x0100		/* Open file and nuke contents */
#define	_O_WRONLY	0x0200		/* Open file for writing */

/*
 *	CREAT/OPEN "mode" cardinal values for cio/tcio.
 */

#define	_S_IREAD	0x0001		/* Allow reading */
#define	_S_IWRITE	0x0002		/* Allow writing */


/*
 *	Portable versions of magic values.
 */

#define	RPC_EOF		-1		/* Portable version of EOF */
#define	RPC_NULL	0		/* Portable version of NULL */
#define	RPC_NOTEOF	1		/* Portable version of !EOF */
#define	RPC_NOTNULL	2		/* Portable version of !NULL */

/*
 *	Structure template for "localtime/gmtime" functions.  This structure
 *	must be completely "packed" (no internal alignment gaps), by the host
 *	"C" compiler to be compatible with the protocol and the Transputer.
 */

struct	rpc_time
	{
	unsigned char	tm_sec;		/* Seconds after the minute [0-59] */
	unsigned char	tm_min;		/* Minutes after the hour [0-59] */
	unsigned char	tm_hour;	/* Hours since midnight [0-23] */
	unsigned char	tm_mday;	/* Day of month [1-31] */
	unsigned char	tm_mon;		/* Months since January [0-11] */
	unsigned char	tm_year;	/* Years since 1900 */
	unsigned char	tm_wday;	/* Days since Sunday [0-6] */
	unsigned char	low_tm_yday;	/* Day of year [0-365] (LOW BYTE) */
	unsigned char	high_tm_yday;	/* Day of year [0-365] (HIGH BYTE) */
	unsigned char	tm_isdst;	/* Daylight Savings Time flag */
	};

#define	RPC_TIME_DATA	10		/*  # of bytes of time data in above */

/*
 *	Function numbers
 */

#define	FN_VCHAN_MODE	0
#define	FN_ARG_GET	1
#define	FN_ARG_SIZE	2
#define	FN_CIOEXT	3
#define	FN_CLEARERR	4
#define	FN_CLOSE	5
#define	FN_CREAT	6
#define	FN_DUP		7
#define	FN_DUP2		8
#define	FN_ERRNO	9
#define	FN_EXIT		10
#define	FN_FCLOSE	11
#define	FN_FCLOSEALL	12
#define	FN_FDOPEN	13
#define	FN_FEOF		14
#define	FN_FERROR	15
#define	FN_FFLUSH	16
#define	FN_FGETC	17
#define	FN_FGETS	18
#define	FN_FILENO	19
#define	FN_FOPEN	20
#define	FN_FPUTC	21
#define	FN_FREAD	22
#define	FN_FREOPEN	23
#define	FN_FSEEK	24
#define	FN_FTELL	25
#define	FN_FWRITE	26
#define	FN_GETCH	27
#define	FN_GETCHE	28
#define	FN_GETENV	29
#define	FN_KBHIT	30
#define	FN_LSEEK	31
#define	FN_OPEN		32
#define	FN_READ		33
#define	FN_REMOVE	34
#define	FN_RENAME	35
#define	FN_SYSTEM	36
#define	FN_TIME		37
#define	FN_TIMEGMT	38
#define	FN_TIMELOC	39
#define	FN_TMPFILE	40
#define	FN_TMPNAM	41
#define	FN_UNGETC	42
#define	FN_WRITE	43
#define	FN_BRKPT	44

/*
 *	Virtual Channel definitions.  Shared by LD-NET, CIO and the
 *	"vchan.c" library file.
 *
 *	Note that the virtual link routing table stuff depends on MAX_LINKS
 *	being 4.  If this isn't the case you will have to modify the
 *	"vchan_download" function in LD-NET AND the code which computes the
 *	next desired output link in the "vchan.c" library file, etc.
 */

#define	MAX_LINKS	4		/* Max # of links/processor */
#if	MAX_LINKS != 4
#error	"MAX_LINKSLOG needs work (see ld-net.c src comments for more changes)"
#endif	/* MAX_LINKS != 4 */
#define	MAX_LINKSLOG	2		/* Log base 2 of MAX_LINKS */

#define	NV_RMEM		1		/* Debug read from Transputer mem */
#define	NV_WMEM		2		/* Debug Write into Transputer mem */
#define	NV_GBRKPT	3		/* Debug "go from breakpoint" */
#define	NV_PMSTART	4		/* Start performance monitoring */
#define	NV_PMPOLL	5		/* Read performance monitoring info */

#define	VC_DEBUGIO	0		/* Reserved debugger channel */
#define	VC_HOSTIO	1		/* Reserved host I/O server channel */
#define	VC_RESV_VCHAN	2		/* First user available channel */

/*
 *	Packet "length" header field bit definitions.
 */

#define	_VCH_CNTL	0x8000		/* "cntl" flag for header flds	*/
#define	_VCH_RRDY	0		/* Routing process ready again	*/
#define	_VCH_IRDY	1		/* Input process ready cntl msg	*/
#define	_VCH_ORDY	2		/* Output process rdy cntl msg	*/
#define	_VCH_EOMP	0x4000		/* Last packet flag "len" field	*/

/*
 *	Packet size limits for Virtual Channels (in bytes).
 */

#define	DEF_APLEN	1024		/* Default for adjacent node msgs */
#define	DEF_RPLEN	1024		/* Default for remote node messages */
#define	MAX_APLEN	16383		/* Maximum for adjacent node msgs */
#define	MAX_RPLEN	16383		/* Maximum for remote node messages */
#define	MIN_APLEN	16		/* Minimum for adjacent node msgs */
#define	MIN_RPLEN	16		/* Minimum for remote node messages */

/*
 *	Packet "channel" header field bit definitions.
 */

#define	_VCH_DEBUG	0x8000		/* Flag for host debug request	*/

/*
 *	The following 8 word structure is used to describe and control a single
 *	H1-compatible virtual channel.  This structure is a software equivalent
 *	of the H1 Virtual Link Control Block microcode structure.
 */
typedef	struct
	{
	int	chan;			/* Destination virtual channel	*/
	int	dest;			/* Destination node address	*/
	char	*in_buf;		/* Input message buffer pointer	*/
	int	in_len;			/* Input message current length	*/
	int	in_max;			/* Input message maximum length	*/
	int	in_ws;			/* Waiting-for-input process	*/
	int	out_ws;			/* Waiting-for-output process	*/
	int	status;			/* Control flags		*/
	} _VLCB;

/*
 *	_VLCB "status" field bit definitions.
 */

#define	_VCS_IRDY	0x0001		/* TRUE/remote input proc ready	*/
#define	_VCS_ORDY	0x0002		/* TRUE/remote output proc rdy	*/
#define	_VCS_IALT	0x0004		/* TRUE/local input via "alt"	*/

/*
 *	Array offset definitions for debug register contents.
 */

#define	_DEBUG_WS	0		/* Workspace */
#define	_DEBUG_PC	1		/* Program Counter */
#define	_DEBUG_A	2		/* Areg */
#define	_DEBUG_B	3		/* Breg */
#define	_DEBUG_C	4		/* Creg */
#define	_DEBUG_FA_LOW	5		/* FAreg low order */
#define	_DEBUG_FA_HIGH	6		/* FAreg high order */
#define	_DEBUG_FA_STAT	7		/* FAreg status */
#define	_DEBUG_FB_LOW	8		/* FBreg low order */
#define	_DEBUG_FB_HIGH	9		/* FBreg high order */
#define	_DEBUG_FB_STAT	10		/* FBreg status */
#define	_DEBUG_FC_LOW	11		/* FCreg low order */
#define	_DEBUG_FC_HIGH	12		/* FCreg high order */
#define	_DEBUG_FC_STAT	13		/* FCreg status */

#define	_DEBUG_REGS	14		/* # of register words in array */
