/*
 *	conc.h
 *
 *	Concurrency/Transputer library include file.
 *
 *	Based on the Transputer concurrency function library designed by
 *	Jeffrey Mock of PIXAR (Version 2).  This implementation by Scott Lloyd
 *	of CSA.  Additional routines provided by Logical Systems (and others).
 */

#ifndef	_CONC				/* Avoid duplication */
#define	_CONC		1

#define	NULL		0

typedef	int	Channel;

typedef	struct				/* Save structure for T4 software FP */
	{
	int	FAreg[2];		/* Simulated FAreg */
	int	FBreg[2];		/* Simulated FBreg */
	int	FCreg[2];		/* Simulated FCreg */
	int	status;			/* Simulated rounding mode status */
	} FPstate;

typedef struct
	{
#ifdef	T212
	int	endp[2];		/* Codespace for ENDP instr */
#else	/* !T212 */
	int	minstk[1];		/* Workspace after last ENDP instr */
	int	endp[1];		/* Codespace for ENDP instr */
#endif	/* T212 */
	int	iptr;			/* Addr of inst after last ENDP inst */
	int	count;			/* Number of processes to join */
	int	wpfinal;		/* Workspace after the join */
	} Forkblk;

typedef	int	PDes;			/* Transputer process descriptor */

typedef	struct
	{
	int	*pdesc;
	void	(*func)();
	void	*ws;
	int	wssize;
	int	nparam;
	} Process;

typedef	struct
	{
	int	use;
	int	*head;
	int	*tail;
	} Semaphore;

#ifdef	T212
#define	NOPROCESS	0x8000
#define	_WORD_SIZE	2
#else	/* !T212 */
#define	NOPROCESS	0x80000000
#define	_WORD_SIZE	4
#endif	/* T212 */

#define	LINK0OUT	((Channel *) (NOPROCESS | (_WORD_SIZE * 0)))
#define	LINK1OUT	((Channel *) (NOPROCESS | (_WORD_SIZE * 1)))
#define	LINK2OUT	((Channel *) (NOPROCESS | (_WORD_SIZE * 2)))
#define	LINK3OUT	((Channel *) (NOPROCESS | (_WORD_SIZE * 3)))
#define	LINK0IN		((Channel *) (NOPROCESS | (_WORD_SIZE * 4)))
#define	LINK1IN		((Channel *) (NOPROCESS | (_WORD_SIZE * 5)))
#define	LINK2IN		((Channel *) (NOPROCESS | (_WORD_SIZE * 6)))
#define	LINK3IN		((Channel *) (NOPROCESS | (_WORD_SIZE * 7)))
#define	EVENT		((int *) (NOPROCESS | (_WORD_SIZE * 8)))
#define	TIMER0		((int *) (NOPROCESS | (_WORD_SIZE * 9)))
#define TIMER1		((int *) (NOPROCESS | (_WORD_SIZE * 10)))
#define	INT_W		((int *) (NOPROCESS | (_WORD_SIZE * 11)))
#define	INT_I		((int *) (NOPROCESS | (_WORD_SIZE * 12)))
#define	INT_A		((int *) (NOPROCESS | (_WORD_SIZE * 13)))
#define INT_B		((int *) (NOPROCESS | (_WORD_SIZE * 14)))
#define INT_C		((int *) (NOPROCESS | (_WORD_SIZE * 15)))
#define INT_STATUS	((int *) (NOPROCESS | (_WORD_SIZE * 16)))
#define	INT_E		((int *) (NOPROCESS | (_WORD_SIZE * 17)))

/*
 *	0xF3FB2141 is "ldc 1, ldpi, endp"!
 */
#ifdef	T212
#define PForkInit(f,c)	{ (f).count = c; (f).endp[0] = 0x2141; \
			  (f).endp[1] = 0xF3FB; }
#else	/* !T212 */
#define PForkInit(f,c)	{ (f).count = c; (f).endp[0] = 0xF3FB2141; }
#endif	/* T212 */
#define PFork(f,p)	{ *((int **)(p)) = (f).endp; \
			  PRun((p) | ProcGetPriority()); }
#define PForkHigh(f,p)  { *((int **)(p)) = (f).endp; PRun(p); }
#define PForkLow(f,p)   { *((int **)(p)) = (f).endp; PRun((p) | 1); }

#define	SEMAPHOREINIT	{0, (int *) NOPROCESS, (int *) NOPROCESS}
#define	SEMINIT(s)	(s).use = 0;

#define	HSemP(s)	(_HSemP(&(s)))
#define	HSemV(s)	(_HSemV(&(s)))

#define	SemP(s)		((s).use =((s).use <= 0) ? ((s).use + 1) : _SemP(&(s)))
#define	SemV(s)		((s).use = ((s).use < 2) ? ((s).use - 1) : _SemV(&(s)))

/* * * * * * * * * * * * Concurrency * * * * * * * * * * * */

void	PHalt(PDes *);
void	_BUILTIN_PHalt(PDes *);
#define	PHalt(p)	_BUILTIN_PHalt(p)
void	PJoin(Forkblk *fblk);
void	PRun(PDes);
void	_BUILTIN_PRun(PDes);
#define	PRun(p)		_BUILTIN_PRun(p)
PDes	PSetup(void *,void (*)(),int,int,...);
void	PStop(void);
void	_BUILTIN_ProcStop(void);
#define	PStop()		_BUILTIN_ProcStop()

Process	*ProcAlloc(void (*)(),int,int,...);
void	ProcFree(Process *);

void	ProcInit(Process *,void (*)(),void *,int,int,...);
void	ProcParam(Process *,...);

void	ProcPar(Process *,...);
void	ProcParList(Process **);
void	ProcPriPar(Process *,Process *);

void	ProcRun(Process *);
void	ProcRunHigh(Process *);
void	ProcRunLow(Process *);

/* * * * * * * * * * * * Communication * * * * * * * * * * * */

Channel	*ChanAlloc(void);
void	ChanFree(Channel *);
int	ChanReset(Channel *);
int	_BUILTIN_ChanReset(Channel *);
#define	ChanReset(ch)	_BUILTIN_ChanReset(ch)
Channel	*VChan(int);

char	ChanInChar(Channel *);
char	_BUILTIN_ChanInChar(Channel *);
#define	ChanInChar(ch)		_BUILTIN_ChanInChar(ch)
int	ChanInInt(Channel *);
int	_BUILTIN_ChanInInt(Channel *);
#define	ChanInInt(ch)		_BUILTIN_ChanInInt(ch)
void	ChanIn(Channel *,void *,int);
void	_BUILTIN_ChanIn(Channel *,void *,int);
#define	ChanIn(ch,c,i)		_BUILTIN_ChanIn(ch,c,i)
char	VChanInChar(Channel *);
int	VChanInInt(Channel *);
void	VChanIn(Channel *,void *,int);
int	VChanVIn(Channel *,void *,int);

void	ChanOutChar(Channel *,int);
void	_BUILTIN_ChanOutChar(Channel *,int);
#define	ChanOutChar(ch,i)	_BUILTIN_ChanOutChar(ch,i)
void	ChanOutInt(Channel *,int);
void	_BUILTIN_ChanOutInt(Channel *,int);
#define	ChanOutInt(ch,i)	_BUILTIN_ChanOutInt(ch,i)
void	ChanOut(Channel *,void *,int);
void	_BUILTIN_ChanOut(Channel *,void *,int);
#define	ChanOut(ch,c,i)		_BUILTIN_ChanOut(ch,c,i)
void	VChanOutChar(Channel *,int);
void	VChanOutInt(Channel *,int);
void	VChanOut(Channel *,void *,int);
void	VChanVOut(Channel *,void *,int);

/* * * * * * * * * * * * Alternation * * * * * * * * * * * */

int	ProcAlt(Channel *,...);
int	ProcAltList(Channel **);
int	VProcAlt(Channel *,...);
int	VProcAltList(Channel **);

int	ProcSkipAlt(Channel *,...);
int	ProcSkipAltList(Channel **);
int	VProcSkipAlt(Channel *,...);
int	VProcSkipAltList(Channel **);

int	ProcTimerAlt(int,Channel *,...);
int	ProcTimerAltList(int,Channel **);
int	VProcTimerAlt(int,Channel *,...);
int	VProcTimerAltList(int,Channel **);

/* * * * * * * * * * * * Semaphores * * * * * * * * * * * */

int	_HSemP(Semaphore *);
int	_HSemV(Semaphore *);
Semaphore	*SemAlloc(void);
void	SemFree(Semaphore *);
int	_SemP(Semaphore *);
int	_SemV(Semaphore *);

/* * * * * * * * * * * * Miscellaneous * * * * * * * * * * * */

int	BitCnt(int);
int	_BUILTIN_BitCnt(int);
#define	BitCnt(i)		_BUILTIN_BitCnt(i)
int	BitRevNBits(int,int);
int	_BUILTIN_BitRevNBits(int,int);
#define	BitRevNBits(i1,i2)	_BUILTIN_BitRevNBits(i1,i2)
int	BitRevWord(int);
int	_BUILTIN_BitRevWord(int);
#define	BitRevWord(i)		_BUILTIN_BitRevWord(i)

void	Move2D(void *src,void *dst,int width,int len,int src_width,
		int dst_width);
void	_BUILTIN_Move2D(void *src,void *dst,int width,int len,int src_width,
		int dst_width);
#define	Move2D(a,b,c,d,e,f)		_BUILTIN_Move2D(a,b,c,d,e,f)
void	Move2DNonZero(void *src,void *dst,int width,int len,int src_width,
		int dst_width);
void	_BUILTIN_Move2DNonZero(void *src,void *dst,int width,int len,
		int src_width,int dst_width);
#define	Move2DNonZero(a,b,c,d,e,f)	_BUILTIN_Move2DNonZero(a,b,c,d,e,f)
void	Move2DZero(void *src,void *dst,int width,int len,int src_width,
		int dst_width);
void	_BUILTIN_Move2DZero(void *src,void *dst,int width,int len,
		int src_width,int dst_width);
#define	Move2DZero(a,b,c,d,e,f)		_BUILTIN_Move2DZero(a,b,c,d,e,f)

void	ProcAfter(int);
void	_BUILTIN_ProcAfter(int);
#define	ProcAfter(i)		_BUILTIN_ProcAfter(i)
int	ProcGetPriority(void);
int	_BUILTIN_ProcGetPriority(void);
#define	ProcGetPriority()	_BUILTIN_ProcGetPriority()
void	ProcReschedule(void);
void	_BUILTIN_ProcReschedule(void);
#define	ProcReschedule()	_BUILTIN_ProcReschedule()
void	ProcStop(void);
void	_BUILTIN_ProcStop(void);
#define	ProcStop()		_BUILTIN_ProcStop()
void	ProcWait(int);
void	_BUILTIN_ProcWait(int);
#define	ProcWait(i)		_BUILTIN_ProcWait(i)

int	ProcToLow(void);
int	ProcToHigh(void);

void	GetHiPriQ(void *results);
void	_BUILTIN_GetHiPriQ(void *results);
#define	GetHiPriQ(r)	_BUILTIN_GetHiPriQ(r)
void	GetLoPriQ(void *results);
void	_BUILTIN_GetLoPriQ(void *results);
#define	GetLoPriQ(r)	_BUILTIN_GetLoPriQ(r)
void	SetHiPriQ(void *front,void *back);
void	_BUILTIN_SetHiPriQ(void *front,void *back);
#define	SetHiPriQ(f,b)	_BUILTIN_SetHiPriQ(f,b)
void	SetLoPriQ(void *front,void *back);
void	_BUILTIN_SetLoPriQ(void *front,void *back);
#define	SetLoPriQ(f,b)	_BUILTIN_SetLoPriQ(f,b)

void	SetTime(int);
void	_BUILTIN_SetTime(int);
#define	SetTime(i)	_BUILTIN_SetTime(i)
int	Time(void);
int	_BUILTIN_Time(void);
#define	Time()		_BUILTIN_Time()

void	restorefp(FPstate *fp);
void	savefp(FPstate *fp);

extern	Channel	*_boot_chan_in;		/* Input channel booted from */
extern	Channel	*_boot_chan_out;	/* Output channel for above */
extern	Channel	*_host_chan_in;		/* Input channel from host */
extern	Channel	*_host_chan_out;	/* Output channel for above */
extern	void	(*_HostIn)(Channel *,void *,int);	/* Host input func */
extern	void	(*_HostOut)(Channel *,void *,int);	/* Host output func */
extern	int	_node_number;		/* Current node # from ".nif" file */

/* * * * * * * * * * * * Reliable Communication * * * * * * * * * * * */

int	ChanOutTimeFail(Channel *,void *,int,int);
int	ChanOutChanFail(Channel *,void *,int,Channel *);

int	ChanInTimeFail(Channel *,void *,int,int);
int	ChanInChanFail(Channel *,void *,int,Channel *);

/* * * * * * * * * * * * Internal Memory Stack * * * * * * * * * * * */

int	ProcCall(int (*)(),void *,int,...);

#endif	/* _CONC */
