
 /*
 File: LDBAL.H
 Version: 1.3
 Header for Load-Balance Library
 (C) Copyright Alta Technology Corporation, 1993
 Written by Scott Lloyd
 4-30-93
 */

#ifndef _LDBAL
#define _LDBAL 1

#include <conc.h>

#define FLSTYP 0
#define JOBTYP 1
#define DATTYP 2

#define ISFLS(n) (n == 0)
#define ISJOB(n) (n & 1)
#define ISDAT(n) (!(n & 1))

#define NOLBPRC (LBPRC *)0

#ifdef _NOMACRO

#define FeedIn(hdr,buf) RecvBal(fin,(Channel *)0,hdr,buf)
#define FeedOut(hdr,buf) SendBal(fout,hdr,buf)
#define JobIn(hdr,buf) RecvBal(jin,jreq,hdr,buf)
#define JobOut(hdr,buf) SendBal(jout,hdr,buf)
#define CollectIn(hdr,buf) RecvBal(cin,(Channel *)0,hdr,buf)
#define CollectOut(hdr,buf) SendBal(cout,hdr,buf)

#else

#define FeedIn(hdr,buf)\
	ChanIn(fin,hdr,(int)sizeof(LBHDR));\
	if ((hdr)->len) ChanIn(fin,buf,(hdr)->len)
#define FeedOut(hdr,buf)\
	ChanOut(fout,hdr,(int)sizeof(LBHDR));\
	if ((hdr)->len) ChanOut(fout,buf,(hdr)->len)
#define JobIn(hdr,buf)\
	ChanOut(jreq,buf,1);\
	ChanIn(jin,hdr,(int)sizeof(LBHDR));\
	if ((hdr)->len) ChanIn(jin,buf,(hdr)->len)
#define JobOut(hdr,buf)\
	ChanOut(jout,hdr,(int)sizeof(LBHDR));\
	if ((hdr)->len) ChanOut(jout,buf,(hdr)->len)
#define CollectIn(hdr,buf)\
	ChanIn(cin,hdr,(int)sizeof(LBHDR));\
	if ((hdr)->len) ChanIn(cin,buf,(hdr)->len)
#define CollectOut(hdr,buf)\
	ChanOut(cout,hdr,(int)sizeof(LBHDR));\
	if ((hdr)->len) ChanOut(cout,buf,(hdr)->len)

#endif /* _NOMACRO */

typedef struct {
	int len;
	int type;
} LBHDR;

typedef struct {
#ifdef ANSI
	void (*proc)(...);
#else
	void (*proc)();
#endif /* ANSI */
	int  wkspace;
	int  bufsize;
} LBPRC;

extern Channel *fin,*fout;
extern Channel *cin,*cout;
extern Channel *jin,*jout,*jreq;

#ifdef ANSI

int  InitBal(LBPRC *feed, LBPRC *job, LBPRC *collect,
             int fjsize, int jcsize, int uplk, int dnlk);
void SendBal(Channel *out, LBHDR *hdr, char *buf);
void RecvBal(Channel *in, Channel *req, LBHDR *hdr, char *buf);

#else

int  InitBal();
void SendBal();
void RecvBal();

#endif /* ANSI */
#endif /* _LDBAL */
