/*---------------------NIF FILE GENERATION PROGRAM-----------------------*/
/*-----------------------------------------------------------------------*/
/* makenif.c                                                             */
/* Nandini Natarajan                                                     */
/* Department of Computer Science                                        */
/* Smith College, Northampton MA 01063                                   */
/* June 16, 1993                                                         */
/*-----------------------------------------------------------------------*/
/* This program can be distributed freely in its unmodified form, and    */
/* must include this header.                                             */
/* The most up-to-date version of this program can be ftp-ed anonymously */
/* from grendel.smith.edu.                                               */
/* Please report any bug or modification to thiebaut@sophia.smith.edu    */
/*-----------------------------------------------------------------------*/
/* This program is written for Turbo C V3.1, IBM PC plateforms.          */
/* To generate the executable version, create a project containing the   */
/* following files:                                                      */
/*                                                                       */
/*          makenif.c                                                    */
/*          cga.obj                                                      */
/*          egavga.obj                                                   */
/*          trip.obj                                                     */
/*          goth.obj                                                     */
/*                                                                       */
/* The *.obj files are generated with the bgiobj utility.                */
/*-----------------------------------------------------------------------*/
/* This program creates the NIF file for a user defined configuration    */
/* of the transputers of a 32-transputer SUPER SET PLUS                  */
/*                                                                       */
/* The primary data structure is an array of 32 records -- each          */
/* containing information (like screen coordinates, color, pattern       */
/* link connections, and reset numbers) about each of the 32 transputers.*/
/*                                                                       */
/* Another array maintains the list of permissible connections between   */
/* the transputers within a cluster and between clusters.  As the program*/
/* executes, and connections are set up, the data structure is updated to*/
/* reflect the changes.                                                  */
/*                                                                       */
/* For example, if link 1 of a transputer is being used,                 */
/* then all other possible connections from link 1 of the particular     */
/* transputer are now made invalid.                                      */
/* The information about the configuration is written to a file at the   */
/* end of the program.  This program has been tested by generating       */
/* several network configurations and feeding the resulting nif file to  */
/* acquire, the software allocating superset transputers to a user.      */
/*                                                                       */
/*-----------------------------------------------------------------------*/
#include <graphics.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

/*---------------------------DEFINITIONS--------------------------------*/

#define TRUE       1
#define FALSE      0
#define MAXTPUTERS 32
#define ENTER      13
#define SPACE      32
#define BACKSPACE  8
#define NOTCONNECT -1
#define CONNECT    1

/*------------------------MACROS---------------------------------------*/
#define lookup(FNode,Flink,ToNode) legalconnect[FNode][Flink][ToNode]
#define iseven(x) (!(x%2))

/*-----------------------STRUCT  DEFINITION-----------------------------*/

typedef struct tputerarray TPUTERARRAY;

struct tputerarray {
        int startx;            /* start coordinates */
        int starty;
        int ifconnect;         /* if node part of network */
        int lastpattern;       /* last pattern and color assigned */
        int lastcolor;
        char link0[10];        /* all 4 link connections of the tputer */
        char link1[10];
        char link2[10];
        char link3[10];
        int resetnum;          /* who resets this tputer */
        int isroot;  /*This field indicates if a tputer is root in a config*/
};
/*----------------------------------------------------------------------*/
/*------------------------FUNCTION PROTOTYPES---------------------------*/
/* List of functions.  Purpose explained within each function           */

void PrintNif(void);
int  GetRawString(int x, int y, char *s, int);
void getstring(int,int,char *);
void drawcluster(void);
void border(void);
void getconfig();
void ShowConfig(void);
void init_legal(void);
void whatconnects(char *,char *);
void linkconnects(char *);
void BlankOut(int,int,int,int);
void figurecoords(char *);
void Initialize(void);
int  empty(char *);
void ConnectToConfig(int,int);
void rootchoice(void);
void drawborders(void);

/*-----------------------------------------------------------------------*/
/*--------------------------GLOBALS VARIABLES----------------------------*/

int legalconnect[MAXTPUTERS+1][4][MAXTPUTERS+1];
                                       /* [FromNode][FromLink][ToNode]  */
TPUTERARRAY t[MAXTPUTERS+1];           /* array of tputers: startx,starty,ifconnect */

int color[] = {
        BLUE,RED,GREEN,CYAN};  /* color of each tputer */

FILE *niffile;

int    GraphDriver;             /* The Graphics device driver           */
int    GraphMode;               /* The Graphics mode value              */
double AspectRatio;             /* Aspect ratio of a pixel on the screen*/
int    MaxX, MaxY;              /* The maximum resolution of the screen */
int    MaxColors;               /* The maximum # of colors available    */
int    ErrorCode;               /* Reports any graphics errors          */
struct palettetype palette;     /* Used to read palette info            */
int endx,endy;

/*----------------------------------------------------------------------*/

main()

{

        Initialize();     /* Set up the Graphics drivers, etc */

        init_legal();     /*initialize the legal connections array */


        setcolor(LIGHTCYAN);

        border();

        drawcluster();    /* Draw the Transputers and the Host computer */

        rootchoice();     /* Ask User to Make the choice of the Root Node */

        getconfig();      /* Get the Configuration until user wants to quit */

        ShowConfig();     /* Delete unconnected nodes, only show config */

        PrintNif();      /* Print to the nif file */

        closegraph();
        return(0);

}
/*-----------------------------------------------------------------------*/
/* PRINTNIF(): This function prints the info about the network to a nif  */
/* file in the right syntax.                                             */
/*-----------------------------------------------------------------------*/

void PrintNif(void)
{
        int i;
        char c;
        char temp[80];
        char progname[20],Nifname[20];

        /* Get the Name of the Program file to be loaded onto the network */

        BlankOut(30,310,630,410);
        outtextxy(30,310,"Name of program to be loaded on transputer(s)?");
        GetRawString(30,320,progname,0);

        /* Get the Name of the .NIF file */

        BlankOut(30,310,630,410);

        outtextxy(30,310,"Output file name (please add .nif extension)?");
        GetRawString(30,320,Nifname,0);

        if ((niffile = fopen(Nifname,"w"))==NULL)
        {
                fprintf(stderr,"Cannot Open Output File\n");
                exit(1);
        }
        fprintf(niffile,"buffer_size\t200;\n");
        fprintf(niffile,"host_server\tcio.exe;\n");
        fprintf(niffile,"level_timeout\t400;\n");
        fprintf(niffile,"decode_timeout\t2000;\n");


        for (i = 1; i <=MAXTPUTERS; i++)
        {
                if (t[i].ifconnect!=NOTCONNECT)
                {
                        sprintf(temp,"\n%d,\t%s,\tR%d,\t%s,\t%s,\t%s,\t%s;",i,progname,
                        t[i].resetnum,t[i].link0,t[i].link1,t[i].link2,t[i].link3);
                        fprintf(niffile,temp);
                }

        }
        fclose(niffile);
}

/*------------------------------------------------------------------------*/
/* SHOWCONFIG(): This function deletes all the unconnected nodes from the */
/* screen, keeping only those in the network.                             */
/*------------------------------------------------------------------------*/

void ShowConfig(void)
{
        unsigned size;
        void * buf;
        int i,posn;
        int numsteps[] = {
                3,0,1,2 };

        setlinestyle(SOLID_LINE,0,NORM_WIDTH);

        for (i = 1; i <=MAXTPUTERS; i++)
        {
                if (t[i].ifconnect == NOTCONNECT)
                {
                        size = imagesize(t[i].startx,t[i].starty,t[i].startx+60,t[i].starty+60);
                        buf = malloc(size);
                        if (buf)
                        {
                                getimage(t[i].startx,t[i].starty,t[i].startx+60,t[i].starty+60,buf);
                                putimage(t[i].startx,t[i].starty,buf,XOR_PUT);
                                free(buf);
                        }
                }

        }
        /* Now, highlight the nodes that are in the network */

        for(i = 1; i <=MAXTPUTERS; i++)
        {
                if (t[i].ifconnect!=NOTCONNECT)
                {
                        posn = numsteps[i%4];
                        rectangle(t[i-posn].startx,t[i-posn].starty,t[i-posn].startx+125,
                        t[i-posn].starty+125);
                }
        }
}

/*------------------------------------------------------------------------*/
/* BORDER(): This function draws the borders around the screen            */
/*------------------------------------------------------------------------*/

void border()

{
        line(0,0,639,0);

        line(0,0,0,479);

        line(0,479,639,479);

        line(639,0,639,479);

        rectangle(10,420,30,440);
        setfillstyle(BKSLASH_FILL,CYAN);
        floodfill(15,425,LIGHTCYAN);
        outtextxy(35,430,"PRESENT SELECTION");
        setfillstyle(XHATCH_FILL,MAGENTA);
        rectangle(10,450,30,470);
        floodfill(15,465,LIGHTCYAN);
        outtextxy(35,460,"ALREADY CONNECTED TO NETWORK");

}

/*------------------------------------------------------------------------*/
/* INITIALIZE(): This function initializes the graphics                   */
/*------------------------------------------------------------------------*/

void Initialize(void)
{
        int xasp, yasp;                       /* Used to read the aspect ratio*/

        /* set-up drivers */
        if (registerbgidriver(EGAVGA_driver) < 0) exit(1);
        if (registerbgifont(triplex_font) < 0) exit(1);
        if (registerbgifont(gothic_font)  < 0) exit(1);
        if (registerbgidriver(CGA_driver) < 0) exit(1);

        GraphDriver = DETECT;                 /* Request auto-detection       */
        initgraph( &GraphDriver, &GraphMode, "" );
        ErrorCode = graphresult();            /* Read result of initialization*/
        if( ErrorCode != grOk ){              /* Error occured during init    */
                printf(" Graphics System Error: %s\n", grapherrormsg( ErrorCode ) );
                exit( 1 );
        }

        getpalette( &palette );               /* Read the palette from board  */
        MaxColors = getmaxcolor() + 1;        /* Read maximum number of colors*/

        MaxX = getmaxx();
        MaxY = getmaxy();                     /* Read size of screen          */

        getaspectratio( &xasp, &yasp );       /* read the hardware aspect     */
        AspectRatio = (double)xasp / (double)yasp; /* Get correction factor   */

}

/*-----------------------------------------------------------------------*/
/* DRAWCLUSTER(): This function sets up the start x and y coords of the  */
/* transputers in the array, and then draws them on the screen.  It also */
/* initializes the other fields of the transputer array, and draws host  */
/*-----------------------------------------------------------------------*/

void drawcluster(void)
{
        int i;
        int startx,starty;
        char tnum[4];

        for(i = 1; i <=MAXTPUTERS; i++)
        {
                t[i].ifconnect  = NOTCONNECT;
                t[i].lastpattern = SOLID_FILL;
                t[i].lastcolor = color[i % 4];
                t[i].isroot = FALSE;
                t[i].link0[0] = '\0';
                t[i].link1[0] = '\0';
                t[i].link2[0] = '\0';
                t[i].link3[0] = '\0';
                t[i].resetnum = MAXTPUTERS;
        }

        /* Set up the start x and y coords of the 32 transputers */

        t[1].startx = 1;    t[1].starty = 1;
        t[2].startx = 65;   t[2].starty = 1;
        t[3].startx = 65;   t[3].starty = 65;
        t[4].startx = 1;    t[4].starty = 65;
        t[5].startx = 135;  t[5].starty = 1;
        t[6].startx = 200;  t[6].starty = 1;
        t[7].startx = 200;  t[7].starty = 65;
        t[8].startx = 135;  t[8].starty = 65;
        t[9].startx = 270;  t[9].starty = 1;
        t[10].startx = 335; t[10].starty = 1;
        t[11].startx = 335; t[11].starty = 65;
        t[12].startx = 270; t[12].starty = 65;
        t[13].startx = 405; t[13].starty = 1;
        t[14].startx = 470; t[14].starty = 1;
        t[15].startx = 470; t[15].starty = 65;
        t[16].startx = 405; t[16].starty = 65;
        t[17].startx = 1;   t[17].starty = 135;
        t[18].startx = 65;  t[18].starty = 135;
        t[19].startx = 65;  t[19].starty = 200;
        t[20].startx = 1;   t[20].starty = 200;
        t[21].startx = 135; t[21].starty = 135;
        t[22].startx = 200; t[22].starty = 135;
        t[23].startx = 200; t[23].starty = 200;
        t[24].startx = 135; t[24].starty = 200;
        t[25].startx = 270; t[25].starty = 135;
        t[26].startx = 335; t[26].starty = 135;
        t[27].startx = 335; t[27].starty = 200;
        t[28].startx = 270; t[28].starty = 200;
        t[29].startx = 405; t[29].starty = 135;
        t[30].startx = 470; t[30].starty = 135;
        t[31].startx = 470; t[31].starty = 200;
        t[32].startx = 405; t[32].starty = 200;

        /* Now draw the transputers */

        for (i = 1; i <=MAXTPUTERS; i++)
        {
                startx = t[i].startx;
                starty = t[i].starty;
                rectangle(startx,starty,startx+60,starty+60);
                setfillstyle(SOLID_FILL, color[i%4]);
                floodfill(startx+20,starty+20,LIGHTCYAN);
                sprintf(tnum,"%d",i);
                outtextxy(startx+30,starty+30,tnum);
        }

        /* Finally Draw The Host */

        rectangle(570,200,630,260);
        setfillstyle(SOLID_FILL,LIGHTMAGENTA);
        floodfill(590,220,LIGHTCYAN);
        outtextxy(590,230,"HOST");

}

/*------------------------------------------------------------------------*/
/* GETCONFIG(): This function loops indefinitely, prompting the user      */
/* to choose two transputers, and setting up the connections between them */
/* If the user presses Q (or q) or X (or x), the function returns to the  */
/* calling procedure.                                                     */
/*------------------------------------------------------------------------*/

void getconfig(void)
{

        char c;
        char tputer1[3],tputer2[3];



        while (1)
        {                               /* Do Forever */
                BlankOut(30,310,630,410);
                outtextxy(30,310,"Enter pair of transputers to connect (Q or X to Quit):");
                getstring(30,320,tputer1);
                if (empty(tputer1)) return; /* if user pressed 'Q' or 'X' or 'q' or 'x'*/
                linkconnects(tputer1);
                getstring(30,330,tputer2);
                if(empty(tputer2)) return;
                figurecoords(tputer1);     /* highlight appropriate transputer */
                figurecoords(tputer2);
                whatconnects(tputer1,tputer2);  /* Finally, find connections */
        }

}

/*------------------------------------------------------------------------*/
/* INIT_LEGAL(): This function initializes the legalconnect               */
/* array, and then proceeds to fill it with the legal connection possible */
/* in a cluster.  The array is set up in the following way:               */
/*------------------------------------------------------------------------

              Ŀ
                                       Node32                      
                                                                   
            ...                                                     
                                                                    
         Ŀ    
                                   Node3                          
        Ŀ    
                                 Node2                           
       ͻ    
                               Node1                            
       Ķ    
                                 LINKS                          
               0           1             2             3        
       Ķ    
       Nodes    3                                            
        1                                                  
        2                                                  
        3                                                  
        .                                                  
        .                                                  
        .                                                  
        32                                                 
       ͼ

------------------------------------------------------------------------*/

void init_legal(void)
{
        int i,j,k;
        /* initialize the entire array to -1 */

        for (i = 1; i <= MAXTPUTERS; i++)
        {
                for (j = 0; j < 4; j++)
                {
                        for (k = 1; k <= MAXTPUTERS; k++)
                                legalconnect[i][j][k] = NOTCONNECT;
                }
        }
        /* Now set up the legal connections */

        /* LINK 0 CONNECTTION FOR T1  */
        for(i = 1; i <=MAXTPUTERS; i++)
        {
                if (i % 4 == 1)
                {
                        for (j = 1; j <=MAXTPUTERS; j++)
                                if ((j %4)==0) {
                                        legalconnect[i][0][j] = 3;
                                        legalconnect[j][3][i] = 0;
                                }
                }
        }

        /* LINK 1 CONNECTION FOR T1 */

        for(i = 1; i <=MAXTPUTERS; i++)
        {
                if (i % 4 == 1)
                {
                        for (j = 1; j <=MAXTPUTERS; j++)
                                if ((j %4 ==2) || (j%4 == 3)) {
                                        legalconnect[i][1][j] = 2;
                                        legalconnect[j][2][i] = 1;

                                }
                }
        }

        /* LINK 2 CONNECTION FOR T1 */
        for (i = 1; i <=MAXTPUTERS; i++)
        {
                if (i % 4 == 1)
                {
                        for (j = 1; j <=MAXTPUTERS; j++)
                                if ((j%4 ==2) || (j%4 == 3)) {
                                        legalconnect[i][2][j] = 1;
                                        legalconnect[j][1][i] = 2;
                                }
                }
        }

        /* LINK 3 CONNECTION FOR T1 */
        for (i = 1; i <=MAXTPUTERS; i++)
        {
                if (i % 4 == 1)
                {
                        for (j = 1; j <=MAXTPUTERS; j++)
                                if ((j%4 == 3) || (j %4==0)) {
                                        legalconnect[i][3][j] = 0;
                                        legalconnect[j][0][i] = 3;
                                }
                }
        }
        /* LINK 0 CONNECTTION FOR T2 */
        for (i = 1; i <=MAXTPUTERS; i++)
        {
                if ( i % 4 == 2)
                {
                        for (j = 1; j <=MAXTPUTERS; j++)
                                if (j % 4 == 3)  {
                                        legalconnect[i][0][j] = 3;
                                        legalconnect[j][3][i] = 0;
                                }
                }
        }
        /* LINK 1 CONNECTION FOR T2 */
        for (i = 1; i <=MAXTPUTERS; i++)
        {
                if (i % 4== 2)
                {
                        for (j = 1; j <=MAXTPUTERS; j++)
                                if (j % 4 == 0) {
                                        legalconnect[i][1][j] = 2;
                                        legalconnect[j][2][i] = 1;
                                }
                }
        }
        /* LINK 2 CONNECTION FOR T2 */
        for (i = 1; i <=MAXTPUTERS; i++)
        {
                if ( i % 4 == 2)
                {
                        for (j = 1; j <=MAXTPUTERS; j++)
                                if (j %4 == 0) {
                                        legalconnect[i][2][j] = 1;
                                        legalconnect[j][1][i] = 2;
                                }
                }
        }
        /*LINK 3 CONNECTION FOR T2 */
        for (i = 1; i <=MAXTPUTERS; i++)
        {
                if ( i % 4 == 2)
                {
                        for (j = 1; j <=MAXTPUTERS; j++)
                                if (( j % 4 == 3) || (j % 4 == 0)) {
                                        legalconnect[i][3][j] = 0;
                                        legalconnect[j][0][i] = 3;
                                }
                }
        }
        /* LINK 1 CONNECTION FOR T3 */
        for (i = 1; i <=MAXTPUTERS; i++)
        {
                if ( i % 4 ==3)
                {
                        for (j = 1; j <= MAXTPUTERS; j++)
                                if (j % 4 == 0) {
                                        legalconnect[i][1][j] = 2;
                                        legalconnect[j][2][i] = 1;
                                }
                }
        }
        /*LINK 2 CONNECTION FOR T3 */
        for (i = 1; i<=MAXTPUTERS; i++)
        {
                if ( i % 4 == 3)
                {
                        for (j = 1; j <=MAXTPUTERS; j++)
                                if (j % 4 == 0 ) {
                                        legalconnect[i][2][j] = 1;
                                        legalconnect[j][1][i] = 2;
                                }
                }
        }
}

/*------------------------------------------------------------------------*/
/* GETSTRING(): This function gets a string from stdin, and reads into a  */
/* character array, displaying the string as it goes along.  There are    */
/* special cases for backspace being pressed, and in case the user wants  */
/* to quit.  Also, if the user enters illegal data, it is not accepted    */
/* something legal is entered.                                            */
/*------------------------------------------------------------------------*/
int GetRawString(int x, int y, char *s, int LookForQuit)
{
     int i=0;
     int c, quit;

     s[i]='\0';

     while (((c = getch())!=ENTER) && (c!=SPACE))
     {
          /* Blank Out The Area ...just in case */
          BlankOut(x,y,x+600,y+60);
          drawborders();
          quit = (c=='Q')||(c=='q')||(c=='X')||(c=='x');
          if (quit&LookForQuit) return 1;
          if (c == BACKSPACE)
          {
                        s[i-1] = '\0';
                        outtextxy(x,y,s);
                        i--;
                        continue;
          }
          s[i++]=c;
          s[i]  = '\0';
          outtextxy(x,y,s);
     }
     return 0;
}

void getstring(int x,int y,char *s)
{
        char c,i;
        int  quit, num, good;

        do
        {
          if (GetRawString(x,y,s,1)==1)
             return;
                num = atoi(s);                  /* Convert the string to an integer */
                good = (num >= 1) & (num<=MAXTPUTERS) ;    /* Legal bounds */
                if (!good)
                {      /* Blank out the incorrect tputer choice */

                        printf("%c",07);
                        BlankOut(x,y,x+40,y+9);

                } /* if */

        } while (!good);
}

/*-------------------------------------------------------------------------*/
/* EMPTY(): This function tests to see if a string is empty.               */
/*-------------------------------------------------------------------------*/
int empty(char * s)
{
        return (s[0]=='\0');
}

/*-------------------------------------------------------------------------*/
/* FIGURECOORDS(): This function highlights the transputer presently       */
/* selected by the user.                                                   */
/*-------------------------------------------------------------------------*/

void figurecoords(char *s)
{
        int tnum;

        int c;

        tnum = atoi(s);

        endx = t[tnum].startx+ 20;
        endy = t[tnum].starty+ 20;
        setfillstyle(BKSLASH_FILL,CYAN);
        floodfill(endx,endy,LIGHTCYAN);  /* floodfill needs points inside poly */

}

/*-------------------------------------------------------------------------*/
/* WHATCONNECTS(): This function determines the permissible connections    */
/* between two transputers, and makes the appropriate changes to the       */
/* transputers data structure.                                             */
/*-------------------------------------------------------------------------*/

void whatconnects(char *t1,char *t2)
{
        int link,i,FoundOne,j,IsGood;
        int ycoord,xcoord,c;
        char linkcombo[25],count[4];
        char NodeLink[10];
        struct fillsettingstype p;
        int FNode = atoi(t1);
        int ToNode = atoi(t2);

        FoundOne = FALSE;
        ycoord = 360;

        /* Find all the link connections possible */

        for (i = 0; i <= 3; i++)
        {
                if ((link = lookup(FNode,i,ToNode))!= NOTCONNECT)

                        /* Here FNode and ToNode represent nodes */
                {
                        outtextxy(40,350,"Select links to be used (0, 1, 2, or 3):");
                        xcoord = 40;
                        sprintf(count,"%1d:",i);
                        outtextxy(40,ycoord,count);
                        xcoord+=40;
                        FoundOne = TRUE;
                        sprintf(linkcombo,"%s-Link%1d TO %s-Link%1d",t1,i,t2,link);
                        outtextxy(xcoord,ycoord,linkcombo);
                        ycoord+=10;
                }

        }
        if (FoundOne==FALSE)

        {  /* Need to reinstate the tputers to original color */

                outtextxy(40,350,"No Connections. Any Key to Continue");
                c = getch();
                getfillsettings(&p);
                setfillstyle(t[FNode].lastpattern,t[FNode].lastcolor);
                floodfill(t[FNode].startx + 20,t[FNode].starty + 20,LIGHTCYAN);
                setfillstyle(t[ToNode].lastpattern,t[ToNode].lastcolor);
                floodfill(t[ToNode].startx + 20,t[ToNode].starty + 20,LIGHTCYAN);
                setfillstyle(p.pattern,p.color);
                return;
        }
        IsGood = 0;
        while (!IsGood)
        {
                c = getch();
                c = (c - '0');   /* Convert the character c to an integer */

                /* Need to Check if the Choice is Legal or Not */

                IsGood = !((lookup(FNode,c,ToNode) == NOTCONNECT)|| (!((c >=0)&&(c <=3))));
                if (!IsGood) printf("%c",7);

        }
        ConnectToConfig(FNode,ToNode); /*Color the Nodes to indicate connection */

        /* Now we remove all connections from that link of FNode to all other
                 permissible destinations as given by the legal connections array.
                 And Vice Versa.
             */
        link  = lookup(FNode,c,ToNode);
        for (j = 1; j <= MAXTPUTERS; j++)
        {
                if (lookup(FNode,c,j)!= NOTCONNECT)
                        lookup(j,link,FNode) = NOTCONNECT;

                lookup(FNode,c,j) = NOTCONNECT;
        }



        for(j = 1; j <=MAXTPUTERS; j++)
        {
                if (lookup(ToNode,link,j)!= NOTCONNECT)
                        lookup(j,c,ToNode) = NOTCONNECT;

                lookup(ToNode,link,j) = NOTCONNECT;
        }

        /* We set the resetnumber of the tputers in question */

        if (ToNode < t[FNode].resetnum) t[FNode].resetnum = ToNode;
        if (FNode < t[ToNode].resetnum) t[ToNode].resetnum = FNode;

        /* Now, we put this info in the t array */
        /* That is, what tputer is connected to which tputers by what links */


        sprintf(NodeLink,"%d[%d]",ToNode,link);
        switch (c)
        {
        case 0:
                strcpy(t[FNode].link0,NodeLink);
                break;
        case 1:
                strcpy(t[FNode].link1,NodeLink);
                break;
        case 2:
                strcpy(t[FNode].link2,NodeLink);
                break;
        case 3:
                strcpy(t[FNode].link3,NodeLink);
                break;
        }

        sprintf(NodeLink,"%d[%d]",FNode,c);
        switch (link)
        {
        case 0:
                strcpy(t[ToNode].link0,NodeLink);
                break;
        case 1:
                strcpy(t[ToNode].link1,NodeLink);
                break;
        case 2:
                strcpy(t[ToNode].link2,NodeLink);
                break;
        case 3:
                strcpy(t[ToNode].link3,NodeLink);
                break;
        }
}

/*-------------------------------------------------------------------------*/
/* CONNECTTOCONFIG(): This function colors the nodes that haven been added */
/* to the configuration.  It also sets the ifconnect field of the tputer   */
/* array to CONNECT.                                                       */
/*-------------------------------------------------------------------------*/

void ConnectToConfig(int FNode,int ToNode)
{
        /* This function sets the chosen tputers to the color that indicates that
             they are now part of the configuration */

        struct fillsettingstype p;
        getfillsettings(&p);
        setfillstyle(XHATCH_FILL,MAGENTA); /* This color indicates "connected" */

        floodfill(t[FNode].startx+20,t[FNode].starty+20,LIGHTCYAN);
        floodfill(t[ToNode].startx+20,t[ToNode].starty+20,LIGHTCYAN);

        t[FNode].lastpattern = t[ToNode].lastpattern = XHATCH_FILL;
        t[FNode].lastcolor = t[ToNode].lastcolor = MAGENTA;
        t[FNode].ifconnect = t[ToNode].ifconnect = CONNECT;

        setfillstyle(p.pattern,p.color);
}

/*-------------------------------------------------------------------------*/
/* ROOTCHOICE(): This function prompts the user for the choice of root.    */
/*-------------------------------------------------------------------------*/

void rootchoice(void)
{
        int rootnum;
        int link0 = 0;
        int link;
        char rootnode[3];
        int i;
        struct fillsettingstype p;

        outtextxy(30,310,"Please Make Your Choice For ROOT Node");



        getstring(30,320,rootnode);
        rootnum = atoi(rootnode);     /* Convert string to integer */


        t[rootnum].isroot = TRUE;   /* Yes, it is the Root Node */
        t[rootnum].resetnum = 0;   /* Reset by the Host */
        strcpy(t[rootnum].link0,"0"); /* It is connected to the Host by Link 0 */


        /* We need to indicate that link 0 of root used up and not usable */

        for (i = 1; i <=MAXTPUTERS; i++)
        {
                if (lookup(rootnum,link0,i)!=NOTCONNECT)
                {
                        link = lookup(rootnum,link0,i);
                        lookup(i,link,rootnum) = NOTCONNECT;
                }
                lookup(rootnum,link0,i) = NOTCONNECT;
        }


        outtextxy(t[rootnum].startx+ 20,t[rootnum].starty+40,"ROOT");

        BlankOut(30,310,630,410);
}

/*-------------------------------------------------------------------------*/
/* BLANKOUT(): This function blanks out a particular part of the screen    */
/* given the start and end coords.  It uses the principle of performing    */
/* an xor of the image with itself, thereby getting a blank screen.        */
/*-------------------------------------------------------------------------*/

void BlankOut(int startx,int starty,int endx,int endy)
{
        unsigned int size;
        void *buf;

        if (endx > getmaxx()) endx = getmaxx()-1;
        if (endy > getmaxy()) endy = getmaxy()-1;
        size = imagesize(startx,starty,endx,endy);
        if (size != 0xFFFF) {
                buf = malloc(size);
                if (buf)
                {
                        getimage(startx,starty,endx,endy,buf);
                        putimage(startx,starty,buf,XOR_PUT);
                }
        }
        free(buf);
}

/*-------------------------------------------------------------------------*/
/* LINKconnects(): This function indicates all the possible connections     */
/* from a particular tputer when it is chosen by the user. It also         */
/* highlights these possible connections by outlining the tputers.         */
/*-------------------------------------------------------------------------*/

void linkconnects(char *s)
{
        int i,j,ycoord;
        char list[100],connects[30],temp[20];
        int startx,starty;
        struct linesettingstype lineinfo;
        int tnum;
        tnum = atoi(s);
        ycoord = 340;

        /* Now, we go through to see what connections are possible from chosen
              tputer to others in the network */

        for (i = 0; i < 4; i++)
        {
                setcolor(MAGENTA);
                list[0] = '\0';
                sprintf(connects,"%d-Link%1d -> ",tnum,i);

                for (j = 1; j <=MAXTPUTERS; j++)
                {
                        if (lookup(tnum,i,j)!=NOTCONNECT)
                        {
                                sprintf(temp,"%d ",j);
                                strcat(list,temp);
                                getlinesettings(&lineinfo);
                                setlinestyle(DASHED_LINE,0,NORM_WIDTH);
                                startx = t[j].startx ;
                                starty = t[j].starty ;
                                rectangle(startx,starty,startx+60,starty+60);
                        }
                }
                outtextxy(30,ycoord,connects);
                outtextxy(130,ycoord,list);
                ycoord+=10;
        }
        setlinestyle(lineinfo.linestyle,lineinfo.upattern,lineinfo.thickness);
        setcolor(LIGHTCYAN);
}

/*-------------------------------------------------------------------------*/
/* DRAWBORDERS(): This function is used to draw the borders around the     */
/* tputers in the original drawing color.  This shows that they have been  */
/* deselected.                                                             */
/*-------------------------------------------------------------------------*/

void drawborders(void)
{
        int i;
        int startx,starty;

        for (i = 1; i <= MAXTPUTERS; i++)
        {
                startx = t[i].startx;
                starty = t[i].starty;
                rectangle(startx,starty,startx+60,starty+60);
        }
}

