/*
 *	alt.c
 *
 *	"ALT" primitives for the Transputer.
 *
 *	Based on the Transputer concurrency function library designed by
 *	Jeffrey Mock of PIXAR (Version 2).  This implementation by Scott Lloyd
 *	of CSA.
 *
 *	Copyright (C) Computer System Architects, 1987.
 */

#include	<inline.h>
#include	<conc.h>

int
ProcAlt(Channel *c1, ...)		/* c2, c3, ..., cN, 0 */
	{
	return (ProcAltList(&c1));
	}

int
ProcAltList(Channel **clist)
	{
	int	reserve;		/* Offset 0 */
	int	idx;			/* Offset 1 */
	int	tmp;			/* Offset 2 */
	Channel	*c;			/* Offset 3 */
	Channel	**next;
	;
	#pragma asm
	ALT
	#pragma endasm
	next = clist;
	while(*next != 0)
		{
		c = *next++;
		#pragma asm
		LDL	[c]
		.LDC	1
		ENBC			;Enable channel
		#pragma endasm
		}
	;
	#pragma asm
	ALTWT
	#pragma endasm
	tmp = 0;
	next = clist;
	while(*next != 0)
		{
	        c = *next++;
		#pragma asm
		LDL	[c]
		.LDC	1
		.LDC	0
		DISC			;Disable channel
		CJ	@L5
		LDL	[tmp]		;Save selected alternative
		STL	[idx]
L5
		#pragma endasm
		tmp++;
		}
	;
	#pragma asm
	ALTEND
	#pragma endasm
	return (idx);
	}

int
ProcSkipAlt(Channel *c1, ...)		/* c2, c3, ..., cN, 0 */
	{
	ProcSkipAltList(&c1);
	}

int
ProcSkipAltList(clist)
	Channel	**clist;
	{
	int	reserve;		/* Offset 0 */
	int	idx;			/* Offset 1 */
	int	tmp;			/* Offset 2 */
	Channel	*c;			/* Offset 3 */
	Channel	**next;
	;
	#pragma asm
	ALT
	#pragma endasm
	next = clist;
	while(*next != 0)
		{
		c = *next++;
		#pragma asm
		LDL	[c]
		.LDC	1
		ENBC			;Enable channel
		#pragma endasm
		}
	;
	#pragma asm
	.LDC	1
	ENBS				;Enable skip
	ALTWT
	#pragma endasm
	tmp = 0;
	next = clist;
	while(*next != 0)
		{
		c = *next++;
		#pragma asm
		LDL	[c]
		.LDC	1
		.LDC	0
		DISC			;Disable channel
		CJ	@L6
		LDL	[tmp]		;Save selected alternative
		STL	[idx]
L6
		#pragma endasm
		tmp++;
		}
	;
	#pragma asm
	.LDC	1
	.LDC	0
	DISS				;Disable skip
	CJ	@L7
	.LDC	-1			;Save -1
	STL	[idx]
L7
	ALTEND
	#pragma endasm
	return (idx);
	}

int
ProcTimerAlt(int time, Channel *c1, ...)	/* c2, c3, ..., cN, 0 */
	{
	ProcTimerAltList(time,&c1);
	}

int
ProcTimerAltList(time, clist)
	int	time;
	Channel	**clist;
	{
	int	reserve;		/* Offset 0 */
	int	idx;			/* Offset 1 */
	int	tmp;			/* Offset 2 */
	Channel	*c;			/* Offset 3 */
	Channel	**next;
	;
	#pragma asm
	TALT
	#pragma endasm
	next = clist;
	while(*next != 0)
		{
		c = *next++;
		#pragma asm
		LDL	[c]
		.LDC	1
		ENBC			;Enable channel
		#pragma endasm
		}
	;
	#pragma asm
	LDL	[time]
	.LDC	1
	ENBT				;Enable timer
	TALTWT
	#pragma endasm
	tmp = 0;
	next = clist;
	while(*next != 0)
		{
		c = *next++;
		#pragma asm
		LDL	[c]
		.LDC	1
		.LDC	0
		DISC			;Disable channel
		CJ	@L8
		LDL	[tmp]		;Save selected alternative
		STL	[idx]
L8
		#pragma endasm
		tmp++;
		}
	;
	#pragma asm
	LDL	[time]
	.LDC	1
	.LDC	0
	DIST				;Disable timer
	CJ	@L9
	.LDC	-1			;Save -1
	STL	[idx]
L9
	ALTEND
	#pragma endasm
	return (idx);
	}
