/*
 *	File:	atan264.c
 *
 *	Implements T800/T414 64 bit floating point "atan2(x,y)".
 *
 *	Based on INMOS supplied code.  Modified by Logical Systems for correct
 *	ANSI "C" setting of "errno" in ERANGE/EDOM situations.
 */

#include	<inline.h>
#include	<math.h>
#include	<stddef.h>

#pragma	asm_expand			/* Recognize EDOM/ERANGE in "asm" */

double
atan2(double x,double y)
	{
	#pragma	asm
#ifdef	T800
;
;	T800 64 bit version
;
	.ext	atan
	.ext	errno
	_asm_ajw(-13)
	.ldc	data
	stl	12
	.ldc	0
	stl	11
	.ldc	0
	stl	10
	.ldc	0
	stl	9
	.ldc	0
	stl	8
	ldlp	[y]
	fpldnldb
	fpnan
	stl	11
	ldlp	[y]
	fpldnldb
	fpnotfinite
	stl	10
	ldlp	[x]
	fpldnldb
	fpnan
	stl	9
	ldlp	[x]
	fpldnldb
	fpnotfinite
	stl	8
	ldl	11
	eqc	0
	cj	@L0
	ldl	9
	cj	@L2
L0:	ldlp	[y]
	ldlp	[x]
	fpldnldb
	fpldnladddb
	ldlp	6
	fpstnldb
L1:	.ldc	EDOM			;Domain error (input is NaN)
	_asm_eval1(&errno);
	stnl	0			;Set "errno" to EDOM
	j	@L12
	.align
;
;	The following floating point constant represents the value of the
;	"undefined" NaN.  The original INMOS code used a value for this which
;	is inconsistent with the latest INMOS Compiler Writers' Guide.  The
;	value has been changed to reflect the one used in the "Guide".
;
data:	.dw	0x00000000		;Was 0x00000010
	.dw	0x7ff00002		;Was 0x7FF00000
	.dw	0x54442d18
	.dw	0x3ff921fb
	.dw	0x54442d18
	.dw	0xbff921fb
	.dw	0x54442d18
	.dw	0x400921fb
L2:	ldl	10
	cj	@L3
	ldl	8
	cj	@L3
	ldl	12
	ldlp	6
	.ldc	8
	move
	j	@L1			;Domain error (both inputs infinite)
L3:	ldlp	[y]
	fpldnldb
	fpldzerodb
	fpeq
	cj	@L9
	ldlp	[x]
	fpldnldb
	fpldzerodb
	fpeq
	cj	@L5
	ldl	12
	ldlp	6
	.ldc	8
	move
	j	@L1			;Domain error (both inputs zero)
L5:	ldlp	[x]
	fpldnldb
	fpldzerodb
	fpgt
	cj	@L6
	ldl	12
	ldnlp	2
	ldlp	6
	.ldc	8
	move
	j	@L12
L6:	ldl	12
	ldnlp	4
	ldlp	6
	.ldc	8
	move
	j	@L12
L9:	ldlp	[y]
	fpldnldb
	fpldzerodb
	fpgt
	cj	@L10
	ldlp	[y]
	ldlp	[x]
	fpldnldb
	fpldnldb
	fpdiv
	ldlp	4
	fpstnldb
	ldl	5
	ldl	4
#ifdef	STATIC_LINK
	ldl	[?slb]
#endif	/* STATIC_LINK */
	call	@atan
	ldlp	6
	fpstnldb
	j	@L12
L10:	ldlp	[y]
	ldlp	[x]
	fpldnldb
	fpldnldb
	fpdiv
	ldlp	0
	fpstnldb
	ldlp	0
	fpldnldb
	fpuabs
	ldlp	2
	fpstnldb
	ldl	3
	ldl	2
#ifdef	STATIC_LINK
	ldl	[?slb]
#endif	/* STATIC_LINK */
	call	@atan
	ldlp	4
	fpstnldb
	ldlp	[x]
	fpldnldb
	fpldzerodb
	fpgt
	cj	@L11
	ldlp	4
	ldl	12
	ldnlp	6
	fpldnldb
	fpldnldb
	fpsub
	ldlp	6
	fpstnldb
	j	@L12
L11:	ldl	12
	ldnlp	6
	ldlp	4
	fpldnldb
	fpldnldb
	fpsub
	ldlp	6
	fpstnldb
L12:	ldlp	6
	fpldnldb
	_asm_ajw(13)
	ret
#else	/* T800 */
;
;	T414 64 bit version (from D700D).  NO attempt is made here to optimize
;	the OCCAM generated code (hope to get to this someday...)
;
	.ext	atan
	.ext	errno
	.ext	_FPstate
	.ext	_r64op
	_asm_ajw(-20)
	.ldc	data
	stl	19
	ldlp	[x]
	ldlp	15
	.ldc	8
	move
	ldlp	[y]
	ldlp	13
	.ldc	8
	move
	ldl	13
	stl	11
	ldl	14
	.ldc	2147483647		;~MOSTNEG
	and
	stl	12
	ldl	15
	stl	9
	ldl	16
	.ldc	2147483647		;~MOSTNEG
	and
	stl	10
	ldl	11
	stl	7
	ldl	12
	stl	8
	ldl	9
	stl	5
	ldl	10
	stl	6
	ldl	12
	.ldc	2146435072		;MSW DRealInf
	gt
	eqc	0
	cj	@L2
	ldl	12
	.ldc	2146435072		;MSW DRealInf
	diff
	eqc	0
	cj	@L1
	ldl	11
	mint
	xor
	mint
	gt
L1:	eqc	0
L2:	eqc	0
	cj	@L7
	ldl	12
	ldl	10
	gt
	eqc	0
	cj	@L4
	ldl	12
	ldl	10
	diff
	eqc	0
	cj	@L3
	ldl	11
	mint
	xor
	ldl	9
	mint
	xor
	gt
L3:	eqc	0
L4:	eqc	0
	cj	@L5
	ldlp	[y]
	ldlp	17
	.ldc	8
	move
	.ldc	EDOM			;Domain error ("y" is NaN)
	_asm_eval1(&errno);
	stnl	0			;Set "errno" to EDOM
	j	@L6
	.align
data:	.dw	0x00000000		;"undefined" NaN
	.dw	0x7ff00002
	.dw	0x54442d18
	.dw	0x3ff921fb
	.dw	0x54442d18
	.dw	0xbff921fb
	.dw	0x54442d18
	.dw	0x400921fb
L5:	ldlp	[x]
	ldlp	17
	.ldc	8
	move
	.ldc	EDOM			;Domain error ("x" is NaN)
	_asm_eval1(&errno);
	stnl	0			;Set "errno" to EDOM
L6:	j	@L24
L7:	ldl	10
	.ldc	2146435072		;MSW DRealInf
	gt
	eqc	0
	cj	@L9
	ldl	10
	.ldc	2146435072		;MSW DRealInf
	diff
	eqc	0
	cj	@L8
	ldl	9
	mint
	xor
	mint
	gt
L8:	eqc	0
L9:	eqc	0
	cj	@L10
	ldlp	[x]
	ldlp	17
	.ldc	8
	move
	.ldc	EDOM			;Domain error ("x" (!"y") is NaN)
	_asm_eval1(&errno);
	stnl	0			;Set "errno" to EDOM
	j	@L24
L10:	ldl	11
	ldl	12
	.ldc	2146435072
	diff
	or
	eqc	0
	cj	@L11
	ldl	9
	ldl	10
	.ldc	2146435072
	diff
	or
	eqc	0
	cj	@L11
	ldl	19
	ldlp	17
	.ldc	8
	move
	.ldc	EDOM			;Domain error ("x" & "y" Inf)
	_asm_eval1(&errno);
	stnl	0			;Set "errno" to EDOM
	j	@L24
L11:	ldl	11
	ldl	12
	or
	eqc	0
	cj	@L19
	ldl	9
	ldl	10
	or
	eqc	0
	cj	@L13
	ldl	19
	ldlp	17
	.ldc	8
	move
	.ldc	EDOM			;Domain error ("x" & "y" zero)
	_asm_eval1(&errno);
	stnl	0			;Set "errno" to EDOM
	j	@L17
L13:	ldl	16
	.ldc	0
	gt
	eqc	0
	cj	@L15
	ldl	16
	eqc	0
	cj	@L14
	ldl	15
	mint
	xor
	mint
	gt
L14:	eqc	0
L15:	eqc	0
	cj	@L16
	ldl	19
	ldnlp	2
	ldlp	17
	.ldc	8
	move
	j	@L17
L16:	ldl	19
	ldnlp	4
	ldlp	17
	.ldc	8
	move
L17:	j	@L24
L19:	ldl	14
	.ldc	0
	gt
	eqc	0
	cj	@L21
	ldl	14
	eqc	0
	cj	@L20
	ldl	13
	mint
	xor
	mint
	gt
L20:	eqc	0
L21:	eqc	0
	cj	@L22
	ldlp	[y]
	stl	0
	.ldc	3
	ldlp	[x]
	ldlp	3
	call	@_r64op
	ldl	4			;MSW of double
	ldl	3			;LSW of double
#ifdef	STATIC_LINK
	ldl	[?slb]
#endif	/* STATIC_LINK */
	call	@atan
	.ldc	_FPstate		;Store FAreg result in 17,18
	stl	17
	ldl	17
	ldnl	1
	stl	18
	ldl	17
	ldnl	0
	stl	17
	j	@L24
L22:	.ldc	0
	ldl	16
	gt
	cj	@L23
	ldlp	7
	stl	0
	.ldc	3
	ldlp	5
	ldlp	3
	call	@_r64op
	ldl	4			;MSW of double
	ldl	3			;LSW of double
#ifdef	STATIC_LINK
	ldl	[?slb]
#endif	/* STATIC_LINK */
	call	@atan
	.ldc	_FPstate		;Store FAreg result in 1,2
	stl	1
	ldl	1
	ldnl	1
	stl	2
	ldl	1
	ldnl	0
	stl	1
	ldl	19
	ldnlp	6
	stl	0
	.ldc	1
	ldlp	1
	ldlp	17
	call	@_r64op
	j	@L24
L23:	ldlp	7
	stl	0
	.ldc	3
	ldlp	5
	ldlp	3
	call	@_r64op
	ldl	4			;MSW of double
	ldl	3			;LSW of double
#ifdef	STATIC_LINK
	ldl	[?slb]
#endif	/* STATIC_LINK */
	call	@atan
	.ldc	_FPstate		;Store FAreg result in 1,2
	stl	1
	ldl	1
	ldnl	1
	stl	2
	ldl	1
	ldnl	0
	stl	1
	ldlp	1
	stl	0
	.ldc	1
	ldl	19
	ldnlp	6
	ldlp	17
	call	@_r64op
L24:	ldlp	17
	.ldc	_FPstate		;Addr of "soft" FAreg
	.ldc	8
	move
	_asm_ajw(20)
	ret
#endif	/* T800 */
	#pragma	endasm
	}
