/*
 *	conc.c
 *
 *	Concurrency primitives for the Transputer.
 *
 *	Based on the Transputer concurrency function library designed by
 *	Jeffrey Mock of PIXAR (Version 2).  This implementation by Scott Lloyd
 *	of CSA.  Modified by Kirk Bailey of Logical Systems.
 *
 *	Copyright (C) Computer System Architects and Logical Systems 1987-1989.
 */

#include	<conc.h>
#include	<string.h>

#define	MINWORK		5		/* Minimum workspace size (words) */

void
ProcInit(Process *p,void (*func)(),void *ws,int wssize,int nparam, ...)
	{
	int			*wsp;
	int			work;

	work = nparam + 2;
	if (work < MINWORK)
		work = MINWORK;
	wsp = (int *) ((char *) ws + wssize) - work;	/* Workspace ptr */
	wsp[1] = (int) p;		/* Process descriptor */
	p->pdesc = wsp;
	p->func = func;
	p->ws = ws;
	p->wssize = wssize;
	p->nparam = nparam;
	memcpy((wsp+2),(((char *) (&nparam)) + sizeof(int)),
		(size_t) (nparam * sizeof(int)));
	}

void
ProcPar(Process *p1, ...)		/* p2, p3, ..., pN, 0 */
	{
	ProcParList(&p1);
	}

void
ProcParam(Process *p, ...)		/* p1, p2, ..., pN */
	{
	memcpy((p->pdesc+2),(((char *) (&p)) + sizeof(int)),
		(size_t) (p->nparam * sizeof(int)));
	}

void
ProcParList(Process **plist)
	{
	int			cnt;
	int			*wsp;
	Process			**next;
	Forkblk			fblk;

	cnt = 1;
	for (next = plist; *next != NULL; next++)
		cnt++;			/* Processes to terminate */
	PForkInit(fblk,cnt);
	for (next = plist; *next != NULL; next++)
		{
		wsp = (*next)->pdesc;
		wsp[-1] = (int) (*next)->func;	/* Start iptr */
		PFork(fblk,(PDes)wsp);
		}
	PJoin(&fblk);
	}

void
ProcPriPar(Process *phigh,Process *plow)
	{
	Forkblk			fblk;

	PForkInit(fblk,3);
	phigh->pdesc[-1] = (int) phigh->func;	/* Start iptr for phigh */
	plow->pdesc[-1] = (int) plow->func;	/* Start iptr for plow */
	PForkHigh(fblk,(PDes)phigh->pdesc);
	PForkLow(fblk,(PDes)plow->pdesc);
	PJoin(&fblk);
	ProcToLow();
	}

void
ProcRun(Process *p)
	{
	int			*wsp;

	wsp = p->pdesc;
	wsp[0] = (int) PStop;		/* Setup return iptr */
	wsp[-1] = (int) p->func;	/* Start iptr */
	PRun((PDes)wsp | ProcGetPriority());
	}

void
ProcRunHigh(Process *p)
	{
	int			*wsp;

	wsp = p->pdesc;
	wsp[0] = (int) PStop;     	/* Setup return iptr */
	wsp[-1] = (int) p->func;	/* Start iptr */
	PRun((PDes)wsp);
	}

void
ProcRunLow(Process *p)
	{
	int			*wsp;

	wsp = p->pdesc;
	wsp[0] = (int) PStop;		/* Setup return iptr */
	wsp[-1] = (int) p->func;	/* Start iptr */
	PRun((PDes)wsp | 1);
	}
