
 /*
 File: LDBAL.C
 Version: 1.3
 Code for Load-Balance Library
 (C) Copyright Alta Technology Corporation, 1993
 Written by Scott Lloyd
 4-30-93
 */

#include <stdlib.h> /* malloc */
#include "ldbal.h"

#define BPW ((int)sizeof(int))	/* bytes per word */
#define BWS (11*BPW)	/* Reqbuf:6+2 R:3 */
#define SWS (22*BPW)	/* Selector:6+4 Alt:4+5 R:3 */
#define AWS (21*BPW)	/* Arbiter:5+4 Alt:4+5 R:3 */

#define loop for (;;)
#define JOBSTK

Channel *fin,*fout;
Channel *cin,*cout;
Channel *jin,*jout,*jreq;


/* InitBal:8+11 ProcAlloc:8+6 malloc:4+1 R:2 = 39 words */

int InitBal(feed, job, collect, fjsize, jcsize, uplk, dnlk)
LBPRC *feed;
LBPRC *job;
LBPRC *collect;
int fjsize;
int jcsize;
int uplk;
int dnlk;
{
	int i,j,k,t,n;
	Channel **ai,**so,**sr;
	Process **ps;
	char *buf;
	void Selector(),Arbiter(),Reqbuf();

	for (t = dnlk, n = 0; t; t >>= 1) if (t & 1) n++;
	ps = (Process **)malloc((n+4)*sizeof(Process *));
	k = 0;

	fin = fout = LINK0IN+uplk;
	if (feed) {
		fout = ChanAlloc();
		buf = (char *)malloc((size_t)feed->bufsize);
		ps[k++] = ProcAlloc(feed->proc,feed->wkspace,1,buf);
	}

	cin = cout = LINK0OUT+uplk;
	if (collect) {
		cin = ChanAlloc();
		buf = (char *)malloc((size_t)collect->bufsize);
		ps[k++] = ProcAlloc(collect->proc,collect->wkspace,1,buf);
	}

	so = (Channel **)malloc((n+2)*sizeof(Channel *));
	sr = (Channel **)malloc((n+2)*sizeof(Channel *));
	buf = (char *)malloc((size_t)fjsize);
	ps[k++] = ProcAlloc(Selector,SWS,4,buf,fout,so,sr);

	ai = (Channel **)malloc((n+2)*sizeof(Channel *));
	buf = (char *)malloc((size_t)jcsize);
	ps[k++] = ProcAlloc(Arbiter,AWS,3,buf,ai,cin);

	j = job != NOLBPRC;
	for (t = dnlk, i = 0; t; t >>= 1, i++) {
		if (t & 1) {
			ai[j] = LINK0IN+i;
			so[j] = ChanAlloc();
			sr[j] = ChanAlloc();
			buf = (char *)malloc((size_t)fjsize);
			ps[k++] = ProcAlloc(Reqbuf,BWS,4,buf,so[j],LINK0OUT+i,sr[j]);
			j++;
		}
	}
	ai[j] = so[j] = sr[j] = (Channel *)0;
	if (job) {
		ai[0] = jout = ChanAlloc();
		so[0] = jin = ChanAlloc();
		sr[0] = jreq = ChanAlloc();
		buf = (char *)malloc((size_t)job->bufsize);
#ifndef JOBSTK
		ProcRunLow(ProcAlloc(job->proc,job->wkspace,1,buf));
#endif
	}

	while (k > 0) ProcRunHigh(ps[--k]);
#ifdef JOBSTK
	if (job) (*job->proc)((Process *)0,buf);
#endif
	ProcStop();
}

void SendBal(out, hdr, buf)
Channel *out;
LBHDR *hdr;
char *buf;
{
	ChanOut(out,hdr,(int)sizeof(LBHDR));
	if (hdr->len) ChanOut(out,buf,hdr->len);
}

void RecvBal(in, req, hdr, buf)
Channel *in;
Channel *req;
LBHDR *hdr;
char *buf;
{
	if (req) ChanOut(req,buf,1);
	ChanIn(in,hdr,(int)sizeof(LBHDR));
	if (hdr->len) ChanIn(in,buf,hdr->len);
}

void Selector(p, buf, in, out, req)
Process *p;
char *buf;
Channel *in;
Channel **out;
Channel **req;
{
	int idx;
	char sig;
	LBHDR hdr;

	loop {
		ChanIn(in,&hdr,(int)sizeof(LBHDR));
		if (hdr.len) ChanIn(in,buf,hdr.len);
		if (ISJOB(hdr.type)) {
			idx = ProcAltList(req);
			ChanIn(req[idx],&sig,1);
			ChanOut(out[idx],&hdr,(int)sizeof(LBHDR));
			if (hdr.len) ChanOut(out[idx],buf,hdr.len);
		} else {
			for (idx = 0; req[idx]; idx++) {
				ChanIn(req[idx],&sig,1);
				ChanOut(out[idx],&hdr,(int)sizeof(LBHDR));
				if (hdr.len) ChanOut(out[idx],buf,hdr.len);
			}
		}
	}
}

void Arbiter(p, buf, in, out)
Process *p;
char *buf;
Channel **in;
Channel *out;
{
	int idx, n;
	LBHDR hdr;

	loop {
		idx = ProcAltList(in);
		ChanIn(in[idx],&hdr,(int)sizeof(LBHDR));
		if (hdr.len) ChanIn(in[idx],buf,hdr.len);
		/* merge flushes and data packs */
		if (ISDAT(hdr.type)) {
			for (n = 0; in[n]; n++) {
				if (n == idx) continue;
				loop {
					ChanIn(in[n],&hdr,(int)sizeof(LBHDR));
					if (hdr.len) ChanIn(in[n],buf,hdr.len);
					if (ISDAT(hdr.type)) break;
					ChanOut(out,&hdr,(int)sizeof(LBHDR));
					if (hdr.len) ChanOut(out,buf,hdr.len);
				}
			}
		}
		/* merge flushes only
		int cnt;
		for (n = 0; in[n]; n++);
		cnt = n;
		...
		if (ISFLS(hdr.type)) {
			if (--cnt == 0) cnt = n;
			else continue;
		}
		*/
		/* merge flushes, filter data packs
		int cnt;
		for (n = 0; in[n]; n++);
		cnt = n;
		...
		if (ISDAT(hdr.type)) {
			if (ISFLS(hdr.type) && --cnt == 0) cnt = n;
			else continue;
		}
		*/
		ChanOut(out,&hdr,(int)sizeof(LBHDR));
		if (hdr.len) ChanOut(out,buf,hdr.len);
	}
}

void Reqbuf(p, buf, in, out, req)
Process *p;
char *buf;
Channel *in;
Channel *out;
Channel *req;
{
	LBHDR hdr;

	loop {
		ChanOut(req,buf,1);
		ChanIn(in,&hdr,(int)sizeof(LBHDR));
		if (hdr.len) ChanIn(in,buf,hdr.len);
		ChanOut(out,&hdr,(int)sizeof(LBHDR));
		if (hdr.len) ChanOut(out,buf,hdr.len);
	}
}
