/*
 *	nsio.c
 *
 *	Primitive "C" "printf"-like routine which sends output out the host
 *	services channel without using any protocol.  These routines are often
 *	used in conjunction with the "_ns_main"/"_ns_vcmain" entrypoints when
 *	the link is being used to dump debug messages to a host display utility
 *	such as	TIO.
 *
 *	Copyright 1988-1991 by Logical Systems.
 */

#include	<stdio.h>
#include	<stdlib.h>
#include	<conc.h>
#include	<inline.h>

#define	TRUE	1
#define	FALSE	0

/*
 *	Write a character to the host services link.
 */
static	void
_putchar(int c)
	{
	#pragma	asm
	ldlp	[c]
	_asm_eval1(&_host_chan_out);	;Output channel pointer address
	ldnl	0
	.ldc	1
	out
	#pragma	endasm
	}

/*
 *	Write a string to the boot link.
 */
static	void
_puts(char *s)
	{
	#pragma	asm
p_loop
	ldl	[s]
	lb				;Load byte
#ifdef	T800
	dup				;Save a copy
	stl	3
#else
	stl	3			;Save a copy
	ldl	3
#endif	/* T800 */
	cj	@done			;If '\0' terminator
	ldl	3			;Fresh copy of byte
	call	@_putchar
	ldl	[s]			;Increment string pointer
	adc	1
	stl	[s]
	j	@p_loop
done
	#pragma	endasm
	}

/*
 *	Convert an integer to a string in any base (2-36)
 */
static	char	*
itob(int n,char *s,int base)
	{
	int			c;
	int			negative;
	char			*p;
	char			*q;
	unsigned int		u;

	if((n < 0) && (base == -10))
		{
		negative = TRUE;
		u = -n;
		}
	else
		{
		negative = FALSE;
		u = n;
		}
	if(base == -10)			/* Signals signed conversion */
		base = 10;
	p = q = s;
	do
		{			/* Generate digits in reverse order */
		if((*p = u % base + '0') > '9')
			*p += ('A' - ('9' + 1));
		++p;
		} while((u /= base) > 0);
	if(negative)
		*p++ = '-';
	*p = '\0';			/* Terminate the string */
	while(q < --p)
		{			/* Reverse the digits */
		c = *q;
		*q++ = *p;
		*p = c;
		}
	return (s);
	}

/*
 *	_ns_exit
 */
void	_ns_exit(void)
	{
	;
	#pragma	asm
	stopp				;Kill process
	#pragma	endasm
	}

/*
 *	_ns_printf
 *
 *	Equivalent to "_printf" except all output goes to the "_host_chan_out"
 *	link unconditionally.
 */

void
_ns_printf(char *arg, ...)
	{
	char			**argp;
	int			base;
	int			c;
	char			*fmt;
	char			s[80];

	argp = &arg;

	fmt = *argp;
	while(c = *fmt++)
		{
		if(c != '%')
			{
			_putchar(c);
			continue;
			}
		while((*fmt >= '0') && (*fmt <= '9'))
			fmt++;
		switch(c = *fmt++)
			{
			case 'l':
				switch(c = *fmt++)
					{
					case 'd':
						base = -10;
						goto prt;
					case 'o':
						base = 8;
						goto prt;
					case 'u':
						base = 10;
						goto prt;
					case 'x':
						base = 16;
						goto prt;
					default:
						_putchar(c);
						continue;
					}
			case 'c':
				_putchar((int) (*++argp));
				continue;
			case 'd':
				base = -10;
				goto prt;
			case 'o':
				base = 8;
				goto prt;
			case 'u':
				base = 10;
				goto prt;
			case 'x':
				base = 16;
		prt:
				_puts(itob((int) (*++argp),s,base));
				continue;
			case 's':
				_puts(*++argp);
				continue;
			default:
				_putchar(c);
				continue;
			}
		}
	}
