/*
 *	proc.c
 *
 *	Alternative process primitives for the Transputer.
 *
 *	These primitives are designed as alternatives to the Transputer process
 *	management library routines designed by Jeff Mock.  These routines
 *	implement a "fork"/"join" paradigm which is optimized for minimum
 *	overhead.
 */

#include	<inline.h>
#include	<conc.h>
#include	<string.h>

#define	MINWORK		5		/* Minimum workspace size (words) */

/*
 *	Deschedule a process and store the terminating process descriptor
 *	at the location pointed to by the parameter.
 */

#undef	PHalt

void
PHalt(PDes *pp)
	{
	#pragma asm
	ldlp	0
	ldpri
	or
	ldl	[pp]
	stnl	0
	stopp
	#pragma endasm
	}

/*
 *	Terminate a process spawned by a previous "PFork", "PForkLow" or
 *	"PForkHigh" operation.  Note that one more "PJoin" operation is usually
 *	performed for each previously performed fork operation.  When the
 *	"last" "PJoin" happens (the N+1 case where there have been N forks),
 *	the calling process continues on as if nothing had happened (ie. IT is
 *	the successor process).
 */
void
PJoin(Forkblk *fblk)
	{
	#pragma asm
	.set	IPTR,2			;Offset to IPTR in Forkblk
	.set	WPFINAL,4		;Offset to WPFINAL in Forkblk
	ldlp	0
	ldl	[fblk]
	stnl	WPFINAL			;Workspace after the join
	.ldc	swwp-here
	ldpi
here
	ldl	[fblk]
	stnl	IPTR			;IP after the last endp
	ldl	[fblk]
	ldnlp	IPTR
	endp
swwp
	ldl	WPFINAL-IPTR		;wpfinal in Forkblk
	gajw
	#pragma endasm
	}

/*
 *	Schedule a process for execution at the current priority.
 */

#undef	PRun

void
PRun(PDes p)
	{
	#pragma asm
	ldl	[p]			;The actual parameter
	runp
	#pragma	endasm
	}

/*
 *	Initialize a "PDes" style process descriptor.  Note that the workspace
 *	used is NOT automatically allocated:  You must externally obtain the
 *	space prior to passing it in as the first parameter!
 */
PDes
PSetup(void *ws, void (*func)(), int wsize, int psize, ...)
	{
	int			work;

	if((int) ws & (sizeof(int) - 1))	/* Force alignment */
		{
		wsize += (((int) ws & (sizeof(int) - 1)) - sizeof(int));
		ws = (void *) (((int) ws & (~(sizeof(int) - 1))) +
			sizeof(int));
		}
	wsize &= (~(sizeof(int) - 1));	/* Force alignment */
	work = psize + 1;
	if (work < MINWORK)
		work = MINWORK;
	ws = (int *) ((int) ws + wsize) - work;
	((int *) ws)[-1] = (int) func;
	((int *) ws)[0] = (int) PStop;	/* Execute if (*func)() returns! */
	bcopy((&psize + 1), ((int *) ws + 1), psize * sizeof(int));
	return ((PDes) ws);
	}

/*
 *	Stop the current process without storing the old process descriptor.
 */

#undef	PStop

void
PStop(void)
	{
	#pragma asm
	stopp
	#pragma endasm
	}
