/************************************************************************/
/*									*/
/*	File:	realloc.c						*/
/*									*/
/*	Allocate, free or change the size of a previous heap memory	*/
/*	allocation.							*/
/*									*/
/*	Uses an optimized boundary tag algorithm with an in-use		*/
/*	overhead of one word per block.					*/
/*									*/
/*	Copyright (C) 1988-1990 by Logical Systems.			*/
/*									*/
/*	Last update:	2/20/90						*/
/*									*/
/************************************************************************/

#define		NOTMALLOC
#include	"../malloc.c"		/* Get dynamic memory types etc. */
#include	<string.h>

void	*
realloc(ptr,size)
	register void	*ptr;
	register size_t	size;
	{
	register size_t	bsize,diff,rbsize;
	register struct	header	*hp,*lefttemp,*righthp;
	int	old_priority;

	if(size == 0)
		{
		free(ptr);
		return (NULL);
		}
	else if(ptr == NULL)
		return (malloc(size));
	else
		{
		if(size <= ((size_t) (sizeof(struct header) +
			sizeof(struct header *) - sizeof(size_t))))
			{
			size = sizeof(struct header) + sizeof(struct header *);
			}
		else
			size += sizeof(size_t);	/* Size and tag overhead */

		if(size & (ALIGNMENT - 1))
			size = (size & (~(ALIGNMENT - 1))) + ALIGNMENT;

		hp = (struct header *)
			((char *) ptr - sizeof(struct header *));
		bsize = hp->size & (~(THISINUSE | PREVINUSE));
		old_priority = ProcToHigh();	/* Protect critical section */
		if(size <= bsize)
/*
 *	New allocation smaller (or the same size), as the old allocation.
 */
			{
			if((diff = bsize - size) >= (sizeof(struct header) +
				sizeof(struct header *)))
/*
 *	Remaining part large enough to return to free pool, do so.
 */
				{
				hp->size &= (PREVINUSE | THISINUSE);
				hp->size |= size;
				hp = (struct header *) ((char *) hp + size);
				hp->size = diff | (THISINUSE | PREVINUSE);
				free((void *) ((char *) hp +
					sizeof(struct header *)));
				}
			}
		else
/*
 *	New allocation is larger than old, either grab next higher block or
 *	start over.
 */
			{
			righthp = (struct header *) ((char *) hp + bsize);
			if((! (righthp->size & THISINUSE)) &&
				((diff = size - bsize) <=
					(rbsize = righthp->size &
						(~(THISINUSE | PREVINUSE)))))
/*
 *	Next block is free AND large enough, add it onto current allocation
 *	and return unused portion if appropriate.
 */
				{
				if((diff = rbsize - diff) <
					(size_t) (sizeof(struct header) +
						sizeof(struct header *)))
/*
 *	Not enough left of upper block to keep, allocate it all.
 */
					{
					hp->size &= (PREVINUSE | THISINUSE);
					hp->size |= (bsize + rbsize);
					(righthp->left)->right =
						righthp->right;
					(righthp->right)->left =
						righthp->left;
					if(_freelist == righthp)
						_freelist = righthp->left;
					*((size_t *) ((char *) righthp +
						rbsize)) |= (THISINUSE |
							PREVINUSE);
					}
				else
/*
 *	Return unused portion of upper block to the free storage pool.
 */
					{
					hp->size &= (PREVINUSE | THISINUSE);
					hp->size |= size;
					hp = (struct header *) ((char *) hp +
						size);
					(righthp->left)->right = hp;
					(righthp->right)->left = hp;
/* May partially overlap */		lefttemp = righthp->left;
					hp->right = righthp->right;
					hp->left = lefttemp;
					if(_freelist == righthp)
						_freelist = hp;
					hp->size = diff | PREVINUSE;
					*((struct header **) ((char *) hp +
						diff -
						sizeof(struct header *))) =
							hp;
					}
				}
			else
/*
 *	Next block is either not free or too small.  We need to allocate a new
 *	block and copy the old contents into it.
 */
				{
				if((hp = (struct header *) malloc(size)) !=
					NULL)
					{
					(void) memcpy((void *) hp,ptr,bsize -
						sizeof(struct header *));
					free(ptr);
					}
				ptr = hp;
				}
			}
		}
	if(old_priority)
		ProcToLow();		/* End of critical section */
	return (ptr);
	}
