/*
 *	sscanf and _scf
 *
 *	Copyright (c) 1988-1992 by Logical Systems
 *
 *	Written by Kirk Bailey
 *
 *	Last update:	1/12/92
 */

#include	<stdio.h>
#include	<stddef.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<ctype.h>
#include	<string.h>
#include	<limits.h>

#define	FALSE		0
#define	TRUE		1
#define	FOREVER		for(;;)		/* Or at least awhile */
#define	FPSIZE		100		/* Max. # of characters in a fp # */

#define	NOMATCH		2		/* Input match failure error */

typedef	int		BOOL;
typedef	unsigned short	USHORT;
typedef	unsigned int	UINT;
typedef	unsigned long	ULONG;

/*
 *	_atol
 *
 *	Helper routine for "_scf" to convert a decimal ASCII value to 
 *	binary and update an associated pointer.
 */
static	long
_atol(char **sptr)
	{
	register	long	i;
	register	char	*p;

	i = 0;
	p = *sptr;
	p--;				/* Ptr was advanced past to begin */
	while(isdigit(*p))
		i = 10 * i + *p++ - '0';
	*sptr = p;
	return (i);
	}

/*
 *	skip_space
 *
 *	Helping function for "matchl", "matche" and "_scf".  Skip whitespace
 *	while updating the associated input character count pointer.
 */
static	void
skip_space(int (*get)(FILE *),int (*put)(int,FILE *),FILE *src,int *c_count)
	{
	register	int	temp;

	while(((temp = (*get)(src)) != EOF) && (isspace(temp)))
		(*c_count)++;
	(void) (*put)(temp,src);
	}

/*
 *	store_s
 *
 *	Helping function for "_scf".  Given a signed integral value to store
 *	into a scanf argument do so.
 */
static	va_list
store_s(va_list vargs,int fconv,long value)
	{
	int			*intptr;
	long			*longptr;
	short			*shortptr;

	switch(fconv)
		{
		case 'h':
			shortptr = (short *) va_arg(vargs,short *);
			*shortptr = value;
			break;
		case 'l':
			longptr = (long *) va_arg(vargs,long *);
			*longptr = value;
			break;
		default:
			intptr = (int *) va_arg(vargs,int *);
			*intptr = value;
			break;
		}
	return (vargs);
	}

/*
 *	store_u
 *
 *	Helping function for "_scf".  Given a unsigned integral value to store
 *	into a scanf argument do so.
 */
static	va_list
store_u(va_list vargs,int fconv,ULONG value)
	{
	UINT			*uintptr;
	ULONG			*ulongptr;
	USHORT			*ushortptr;

	switch(fconv)
		{
		case 'h':
			ushortptr = (USHORT *) va_arg(vargs,USHORT *);
			*ushortptr = value;
			break;
		case 'l':
			ulongptr = (ULONG *) va_arg(vargs,ULONG *);
			*ulongptr = value;
			break;
		default:
			uintptr = (UINT *) va_arg(vargs,UINT *);
			*uintptr = value;
			break;
		}
	return (vargs);
	}

/*
 *	isbdigit
 *
 *	Helping function for "matchl".  Determine if a character can be
 *	represented in the given base.
 */
static	BOOL
isbdigit(int digit,int base)
	{
	if(base <= 10)
		{
		if((digit >= '0') && (digit < ('0' + base)))
			return (TRUE);
		else
			return (FALSE);
		}
	else
		{
		digit = (islower(digit) ? toupper(digit) : digit);
		if(isdigit(digit) ||
			((digit >= 'A') && (digit < ('A' + base - 10))))
			{
			return (TRUE);
			}
		else
			return (FALSE);
		}
	}

/*
 *	tobdigit
 *
 *	Helping function for "matchl".  Convert an ASCII character which is
 *	assumed to lie within the base range of 2-36 into the corresponding
 *	binary equivalent.
 */
static	int
tobdigit(int digit)
	{
	if(isdigit(digit))
		return (digit - '0');
	else
		return ((islower(digit) ? toupper(digit) : digit) - 'A' + 10);
	}

/*
 *	readc
 *
 *	Helping function for "matchl".  Reads a character and updates the
 *	specified character count.
 */
static	int
readc(int (*get)(FILE *),FILE *src,int *c_count)
	{
	(*c_count)++;
	return ((*get)(src));
	}

#ifndef	T212
/*
 *	matche
 *
 *	Helping function for "_scf".  Convert a floating point numeric string
 *	in the specified base to the double precision binary equivalent.
 *
 *	Parameters:
 *			"get"		The routine to call to read a char.
 *			"put"		The routine to call to "unget" a char.
 *			"src"		Parameter to get/put.
 *			"width"		The maximum field width (-1L if none).
 *			"c_count"	A pointer to the int which counts the
 *					characters which have been read.
 *			"result"	A pointer to where to store the result.
 */
static	int
matche(int (*get)(FILE *),int (*put)(int,FILE *),FILE *src,long width,
	int *c_count,double *result)
	{
	BOOL			a_digit;
	char			buf[FPSIZE + 1];
	register	int	c;
	register	char	*cptr;
	BOOL			done;
	int			old_count;
/*
 *	Initialize stuff and skip leading whitespace.
 */
	if((width < 0L) || (width > FPSIZE))	/* If not specified or large */
		width = FPSIZE;
	*result = 0.0;			/* Init result in case small field */

	skip_space(get,put,src,c_count);
	old_count = *c_count;		/* # of characters currently eaten */
/*
 *	Process sign indication for number (if positive or signed).
 */
	cptr = buf;
	if((*c_count - old_count) >= width)
		return (FALSE);		/* Max. allowed field width */
	if((c = readc(get,src,c_count)) == '-')
		{
		*cptr++ = c;
		if((*c_count - old_count) >= width)
			return (FALSE);	/* Max. allowed field width */
		c = readc(get,src,c_count);
		}
	else if(c == '+')
		{
		*cptr++ = c;
		if((*c_count - old_count) >= width)
			return (FALSE);	/* Max. allowed field width */
		c = readc(get,src,c_count);
		}
/*
 *	Process fraction up to the (optional), decimal point.
 */
	done = a_digit = FALSE;
	FOREVER
		{
		while(isdigit(c))
			{
			a_digit = TRUE;
			*cptr++ = c;
			c = readc(get,src,c_count);
			if((*c_count - old_count) > width)
				{
				done = TRUE;
				break;
				}
			}
		if(done)
			break;
/*
 *	See if we got a decimal point, if so continue accumulating fraction.
 */
		if(c == '.')
			{
			*cptr++ = c;
			c = readc(get,src,c_count);
			if((*c_count - old_count) > width)
				break;
			while(isdigit(c))
				{
				a_digit = TRUE;
				*cptr++ = c;
				c = readc(get,src,c_count);
				if((*c_count - old_count) > width)
					{
					done = TRUE;
					break;
					}
				}
			}
		if(done)
			break;
/*
 *	See if we have an exponent and process exponent sign.
 */
		if(toupper(c) == 'E')
			{
			*cptr++ = c;
			c = readc(get,src,c_count);
			if((*c_count - old_count) > width)
				break;
			if(c == '-')
				{
				*cptr++ = c;
				c = readc(get,src,c_count);
				if((*c_count - old_count) > width)
					break;
				}
			else if(c == '+')
				{
				*cptr++ = c;
				c = readc(get,src,c_count);
				if((*c_count - old_count) > width)
					break;
				}
/*
 *	Process exponent value if one is present.
 */
			while(isdigit(c))
				{
				*cptr++ = c;
				c = readc(get,src,c_count);
				if((*c_count - old_count) > width)
					break;
				}
			}
		break;
		}
/*
 *	Verify that number was converted OK.
 */
	(void) (*put)(c,src);		/* Put lookahead char back */
	(*c_count)--;
	if((*c_count == old_count) || (! a_digit))
		{
		if(c == EOF)
			return (EOF);
		else
			return (NOMATCH);
		}
/*
 *	We've parsed the number, use "atof" to convert it into binary form.
 */
	*cptr = '\0';
	*result = atof(buf);
	return (FALSE);
	}
#endif	/* !T212 */

/*
 *	matchl
 *
 *	Helping function for "_scf".  Convert a numeric string in the
 *	specified base to a signed or unsigned long as directed.
 *
 *	Parameters:
 *			"get"		The routine to call to read a char.
 *			"put"		The routine to call to "unget" a char.
 *			"src"		Parameter to get/put.
 *			"width"		The maximum field width (-1L if none).
 *			"c_count"	A pointer to the int which counts the
 *					characters which have been read.
 *			"result"	A pointer to where to store the result.
 *			"base"		Base of input number (ala "strtol").
 *			"to_unsigned"	TRUE if result is to be unsigned.
 */
static	int
matchl(int (*get)(FILE *),int (*put)(int,FILE *),FILE *src,long width,
	int *c_count,ULONG *result,int base,BOOL to_unsigned)
	{
	BOOL			a_digit;
	register	int	c;
	BOOL			done;
	BOOL			lead_zero;
	int			old_count;
	ULONG			over_div;
	ULONG			over_rem;
	int			sign;
	register	ULONG	ul;
/*
 *	Initialize stuff and skip leading whitespace.
 */
	if(width < 0L)			/* If not specified */
		width = LONG_MAX;
	ul = 0;				/* Result accumulator */
	*result = 0L;			/* Init result in case small field */

	skip_space(get,put,src,c_count);
	old_count = *c_count;		/* # of characters currently eaten */
/*
 *	Process sign indication for number (if positive or signed).
 */
	if((*c_count - old_count) >= width)
		return (FALSE);		/* Max. allowed field width */
	if((c = readc(get,src,c_count)) == '-')
		{
		sign = -1;
		if((*c_count - old_count) >= width)
			return (FALSE);	/* Max. allowed field width */
		c = readc(get,src,c_count);
		}
	else if(c == '+')
		{
		sign = 1;
		if((*c_count - old_count) >= width)
			return (FALSE);	/* Max. allowed field width */
		c = readc(get,src,c_count);
		}
	else	sign = 1;
/*
 *	See if we have a leading zero.
 */
	if(c == '0')
		{
		lead_zero = TRUE;
		if((*c_count - old_count) >= width)
			return (FALSE);	/* Max. allowed field width */
		c = readc(get,src,c_count);
		}
	else
		lead_zero = FALSE;
/*
 *	Check for hexadecimal prefix ("0x" or "0X").
 */
	if(lead_zero && ((base == 0) || (base == 16)) &&
		((c == 'x') || (c == 'X')))
		{
		lead_zero = FALSE;
		base = 16;		/* We know its hexadecimal now */
		if((*c_count - old_count) >= width)
			return (FALSE);	/* Max. allowed field width */
		c = readc(get,src,c_count);
		}
/*
 *	Check for octal prefix (leading "0").
 */
	else if(lead_zero && (base == 0))
		base = 8;		/* We know its octal */
/*
 *	If the base is still unknown it must be decimal.
 */
	else if(base == 0)
		base = 10;		/* Must be decimal */
/*
 *	We've determined the base, now convert the number.  First compute the
 *	overflow limit.
 */
	if(to_unsigned)
		{
		over_div = ULONG_MAX / base;
		over_rem = ULONG_MAX % base;
		}
	else if(sign > 0)
		{
		over_div = LONG_MAX / base;
		over_rem = LONG_MAX % base;
		}
	else
		{
		over_div = (((ULONG) LONG_MAX) + 1) / base;
		over_rem = (((ULONG) LONG_MAX) + 1) % base;
		}
	a_digit = lead_zero;		/* Make sure at least one digit */
	done = FALSE;
	while(isbdigit(c,base))
		{
		a_digit = TRUE;		/* Had at least one digit */
		if(! done)
			{
			c = tobdigit(c);
			if((ul > over_div) ||
				((ul == over_div) && (c > over_rem)))
				{
				if(to_unsigned)
					*result = ULONG_MAX;
				else if(sign > 0)
					*result = LONG_MAX;
				else
					*result = LONG_MIN;
				done = TRUE;
				}
			else
				ul = (ul * base) + c;
			}
		c = readc(get,src,c_count);
		if((*c_count - old_count) > width)
			break;		/* Max. allowed field width */
		}
	if(! done)
		{
		if(sign < 0)
/*lint -e501 lint doesn't like complementing unsigned values */
			ul = -ul;
/*lint +e501 lint doesn't like complementing unsigned values */
		*result = ul;
		}
	(void) (*put)(c,src);
	(*c_count)--;
	if((*c_count == old_count) || (! a_digit))	/* Match anything? */
		{
		if(c == EOF)
			return (EOF);
		else
			return (NOMATCH);
		}
	return (FALSE);
	}

/*
 *	inscan
 *
 *	Helping function for "_scf".  Determines if a character is present in a
 *	string which is delimited by first and last+1 pointers.
 */
static	BOOL
inscan(char *start,char *stop,int c)
	{
	while(start < stop)
		{
		if(*start++ == c)
			return (TRUE);
		}
	return (FALSE);
	}

/*
 *	_scf
 *
 *	Conversion routine for fscanf,scanf and sscanf.  Conforms to draft
 *	ANSI standard specification.
 */
int
_scf(int (*get)(FILE *),int (*put)(int,FILE *),FILE *src,char *format,
	va_list vargs)
	{
	int			a_count;
	BOOL			assign;
	register	int	c;
	int			c_count;
	BOOL			complement;
	int			done_status;
	int			fconv;
	int			*intptr;
	BOOL			literal;
	long			ltemp;
	register	char	*s;
	char			*start;
	register	int	temp;
	ULONG			ultemp;
	long			width;
#ifndef	T212
	double			dtemp;
	double			*doubleptr;
	float			*floatptr;
#endif	/* !T212 */

	done_status = FALSE;
	a_count = c_count = 0;

	while((c = *format++) != '\0')
		{
		literal = FALSE;
/*
 *	Handle whitespace format directive.
 */
		if(isspace(c))
			{
			while(isspace(c = *format))
				format++;	/* Skip format whitespace */
			while(isspace(c = (*get)(src)))
				c_count++;
			if(c == EOF)
				{
				done_status = EOF;
				break;
				}
			(void) (*put)(c,src);
			}
/*
 *	Handle conversion format directives.
 */
		else if(c == '%')
			{
			c = *format++;	/* Skip '%' */
/*
 *	Check for assignment suppression.
 */
			if(c == '*')
				{
				assign = FALSE;
				c = *format++;
				}
			else
				assign = TRUE;
/*
 *	Check for maximum field width.
 */
			if(isdigit(c))
				{
				width = _atol(&format);
				c = *format++;
				}
			else
				width = -1L;
/*
 *	Check for size modifiers for results.
 */
			fconv = '\0';
			if(strchr("hlL",c) != NULL)
				{
				temp = c;
				c = *format++;
				switch(temp)
					{
					case 'h':
						if(strchr("dinouxX",c) == NULL)
							break;
						fconv = 'h';
						break;
					case 'l':
						if(strchr("deEfgGinouxX",c) ==
							NULL)
							{
							break;
							}
						fconv = 'l';
						break;
					case 'L':
						if(strchr("eEfgG",c) == NULL)
							break;
						fconv = 'L';
						break;
					}
				if(fconv == '\0')
					return (a_count);
				}
/*
 *	Process format conversion specifier.
 */
			switch(c)
				{
				case 'c':
					if(width < 0L)
						width = 1L;
					if(assign)
						{
						s = (char *) va_arg(vargs,
							char *);
						}
					while(width-- > 0L)
						{
						if((temp = (*get)(src)) == EOF)
							{
							done_status = EOF;
							break;
							}
						c_count++;
						if(assign)
							*s++ = temp;
						}
					if(done_status)
						break;
					if(assign)
						a_count++;
					break;
				case 'd':
					if(done_status = matchl(get,put,src,
						width,&c_count,
							(ULONG *) &ltemp,10,
								FALSE))
						{
						break;
						}
					if(assign)
						{
						vargs = store_s(vargs,fconv,
							ltemp);
						a_count++;
						}
					break;
#ifndef	T212
				case 'e':
				case 'E':
				case 'f':
				case 'g':
				case 'G':
					if(done_status = matche(get,put,src,
						width,&c_count,
							(double *) &dtemp))
						{
						break;
						}
					if(assign)
						{
						if((fconv == 'l') ||
							(fconv == 'L'))
							{
							doubleptr = (double *)
								va_arg(vargs,
								double *);
							*doubleptr = dtemp;
							}
						else
							{
							floatptr = (float *)
								va_arg(vargs,
								float *);
							*floatptr = dtemp;
							}
						a_count++;
						}
					break;
#endif	/* !T212 */
				case 'i':
					if(done_status = matchl(get,put,src,
						width,&c_count,
							(ULONG *) &ltemp,0,
								FALSE))
						{
						break;
						}
					if(assign)
						{
						vargs = store_s(vargs,fconv,
							ltemp);
						a_count++;
						}
					break;

				case 'n':
					if(assign)
						{
						intptr = (int *)
							va_arg(vargs,int *);
						*intptr = c_count;
						}
					break;

				case 'o':
					if(done_status = matchl(get,put,src,
						width,&c_count,
							(ULONG *) &ltemp,8,
								TRUE))
						{
						break;
						}
					if(assign)
						{
						vargs = store_s(vargs,fconv,
							ltemp);
						a_count++;
						}
					break;

				case 'p':
				case 'x':
				case 'X':
					if(done_status = matchl(get,put,src,
						width,&c_count,
							(ULONG *) &ltemp,16,
								TRUE))
						{
						break;
						}
					if(assign)
						{
						vargs = store_s(vargs,fconv,
							ltemp);
						a_count++;
						}
					break;

				case 's':
					if(width < 0L)
						width = LONG_MAX;
					if(assign)
						{
						s = (char *) va_arg(vargs,
							char *);
						}
					skip_space(get,put,src,&c_count);
					done_status = EOF;
					while(((temp = (*get)(src)) != EOF) &&
						(! isspace(temp)) &&
							(width-- > 0L))
						{
						c_count++;
						done_status = FALSE;
						if(assign)
							*s++ = temp;
						}
					if(done_status)
						break;
					(*put)(temp,src);
					if(assign)
						{
						*s = '\0';
						a_count++;
						}
					break;

				case 'u':
					if(done_status = matchl(get,put,src,
						width,&c_count,
							&ultemp,10,TRUE))
						{
						break;
						}
					if(assign)
						{
						vargs = store_u(vargs,fconv,
							ultemp);
						a_count++;
						}
					break;

				case '[':
					if(width < 0L)
						width = LONG_MAX;
					if(assign)
						{
						s = (char *) va_arg(vargs,
							char *);
						}
					if((c = *format) == '^')
						{
						complement = TRUE;
						format++;	/* Skip '^' */
						}
					else
						complement = FALSE;
					done_status = EOF;
					if(*format == '\0')
						break;
					start = format;
					while((*(++format) != ']') &&
						(*format != '\0'))
						;
					if(*format == '\0')
						break;
					while(((temp = (*get)(src)) != EOF) &&
						(width-- > 0L))
						{
						if(complement)
							{
							if(inscan(start,format,
								temp))
								{
								break;
								}
							}
						else
							{
							if(! inscan(start,
								format,temp))
								{
								break;
								}
							}
						c_count++;
						done_status = FALSE;
						if(assign)
							*s++ = temp;
						}
					if(done_status)
						{
						if(temp == EOF)
							done_status = EOF;
						else
							done_status = NOMATCH;
						break;
						}
					(*put)(temp,src);
					if(assign)
						{
						*s = '\0';
						a_count++;
						}
					format++;	/* Skip ']' */
					break;

				case '%':
					literal = TRUE;
					break;
				default:
					return(a_count);
				}
			if(done_status)
				break;
			}
		else
			literal = TRUE;
/*
 *	Match literal format directives.
 */
		if(literal)
			{
			if((temp = (*get)(src)) == EOF)
				{
				done_status = EOF;
				break;
				}
			if(temp != c)
				{
				(*put)(temp,src);
				done_status = NOMATCH;
				break;
				}
			c_count++;
			}
		}
/*
 *	Figure out return value based on type of termination and # of assigned
 *	results.
 */
	if((done_status == EOF) && (a_count == 0))
		return (EOF);
	else
		return (a_count);
	}

/*
 *
 *	scf_help1
 *
 *	Helper routine for "sscanf" which "reads" input characters from the
 *	string.
 */
static	int
scf_help1(FILE *s)
	{
	int			rval;

	if((rval = **((char **) s)) == '\0')
		return (EOF);
	(*((char **) s))++;
	return (rval);
	}

/*
 *
 *	scf_help2
 *
 *	Helper routine for "sscanf" which "unreads" input characters from the
 *	string (like "ungetc" for files).  
 */
static	int
scf_help2(int c,FILE *s)
	{
	if(c != EOF)
		(*((char **) s))--;
	return (c);
	}

/*
 *	sscanf
 */
int
sscanf(char *s,const char *format,...)
	{
	int			rval;
	char			*sptr;
	va_list			vargs;

	va_start(vargs,format);		/* Start of var args stuff */
	sptr = s;
	rval = _scf(scf_help1,scf_help2,(FILE *) &sptr,format,vargs);
	va_end(vargs);			/* Cleanup for var args stuff */
	return (rval);
	}
