/*
 *	tstream.c
 *
 *	"stream" I/O routines for client (Transputer) side of "C" remote file
 *	I/O server.  Uses the "tcio.c" I/O primitives.
 *
 *	Copyright 1988-1991 by Logical Systems.
 */

#include	<stdio.h>
#include	<string.h>
#include	<cio.h>

#define	UCHAR	unsigned char

extern	void	_enc_strm(UCHAR *,FILE *); /* Convert strm to 2 bytes	*/
extern	void	_enc_2b(UCHAR *,int);	/* Convert int to 2 bytes	*/
extern	void	_get_str(UCHAR *,int);	/* Get # of bytes from host	*/
extern	int	_get_2b(void);		/* Get 2 bytes from host	*/
extern	int	_get_2beof(void);	/* Get 2 bytes/EOF from host	*/
extern	void	_put_str(UCHAR *,int);	/* Send "raw" string to host	*/
extern	void	_request_errno(void);	/* Get host "errno" value	*/
extern	void	_send_cmd(UCHAR *,int,int); /* Send command to host	*/
extern	int	_unlock_int(int);	/* Unlock client/server channel	*/
extern	char	*_unlock_str(char *);	/* Unlock client/server channel	*/

int
fgetc(FILE *stream)
	{
	UCHAR			buf[2];

	_enc_strm(buf,stream);
	_send_cmd(buf,FN_FGETC,2);
	return (_unlock_int(_get_2beof()));
	}

char	*
fgets(char *s,int n,FILE *stream)
	{
	UCHAR			buf[4];
	int			length;

	if((n < 2) || (n > MAX_MSG_DATA))
		return (NULL);
	_enc_strm(buf,stream);
	_enc_2b(&buf[2],n);
	_send_cmd(buf,FN_FGETS,4);

	if((length = _get_2b()) == RPC_NULL)	/* Figure out if string/NULL */
		{
		_request_errno();
		return (_unlock_str(NULL));
		}
	else				/* Real string, read */
		{
		_get_str((UCHAR *) s,length);
		return (_unlock_str(s));
		}
	}

int
fputc(int c,FILE *stream)
	{
	UCHAR			buf[3];

	_enc_strm(buf,stream);
	buf[2] = c;
	_send_cmd(buf,FN_FPUTC,3);
	return (_unlock_int(_get_2beof()));
	}

int
fputs(const char *s,FILE *stream)
	{
	UCHAR			buf[6];
	int			length;

	if((s == NULL) || ((length = strlen(s)) > MAX_MSG_DATA))
		return (EOF);
	_enc_strm(buf,stream);
	_enc_2b(&buf[2],1);
	_enc_2b(&buf[4],length);
	_send_cmd(buf,FN_FWRITE,6);
	_put_str((UCHAR *) s,length);
	if(_get_2b() != length)
		{
		_request_errno();
		return (_unlock_int(EOF));
		}
	else
		return (_unlock_int(0));
	}

int
getch(void)
	{
	_send_cmd(NULL,FN_GETCH,0);
	return (_unlock_int(_get_2b()));
	}

int
getche(void)
	{
	_send_cmd(NULL,FN_GETCHE,0);
	return (_unlock_int(_get_2b()));
	}

char	*
gets(char *s)
	{
	size_t			length;

	if(fgets(s,MAX_MSG_DATA,stdin) == NULL)
		return (NULL);
	if((length = strlen(s)) > 0)
		{
		if(s[length - 1] == '\n')
			s[length - 1] = '\0';
		}
	return (s);
	}

int
kbhit(void)
	{
	_send_cmd(NULL,FN_KBHIT,0);
	return (_unlock_int(_get_2b()));
	}

int
puts(const char *s)
	{
	if(fputs(s,stdout) == EOF)
		return (EOF);

	if(fputc('\n',stdout)  == EOF)
		return (EOF);

	return (0);
	}
