/*
 *	vchanalt.c
 *
 *	"ALT" primitives for with virtual channels on the Transputer.
 *
 *	Based on the Transputer concurrency function library designed by
 *	Jeffrey Mock of PIXAR (Version 2).  This implementation by Kirk Bailey
 *	based on previous work by Scott Lloyd of CSA.
 *
 *	Copyright 1991 by Logical Systems.
 */

#include	<inline.h>
#include	<conc.h>
#include	<cio.h>

int
VProcAlt(Channel *c1, ...)		/* c2, c3, ..., cN, 0 */
	{
	return (VProcAltList(&c1));
	}

int
VProcAltList(Channel **clist)
	{
/*
 *	Note that "reserve" MUST be listed first so that it is assigned
 *	workspace offset zero.  The ALTWT will clobber this location...
 */
	int			reserve;	/* Offset 0 */

	Channel			*c;
	int			idx;
	Channel			**next;
	int			old_priority;
	int			tmp;

	old_priority = ProcToHigh();	/* Change to high priority */

	#pragma asm
	ALT
	#pragma endasm
	next = clist;
	while(*next != 0)
		{
		if(((int) (c = *next++)) & 1)
			{
			c = (Channel *) (((char *) c) - 1);
			if(((_VLCB *) c)->status & _VCS_ORDY)
				((_VLCB *) c)->in_ws = NULL;
			else
				{
				((_VLCB *) c)->status = ((_VLCB *) c)->status |
					_VCS_IALT;
				}
			c = (Channel *) &(((_VLCB *) c)->in_ws);
			}
		;
		#pragma asm
		LDL	[c]
		.LDC	1
		ENBC			;Enable channel
		#pragma endasm
		}
	;
	#pragma asm
	ALTWT
	#pragma endasm
	tmp = 0;
	next = clist;
	while(*next != 0)
		{
		if(((int) (c = *next)) & 1)
			{
			c = (Channel *) (((char *) c) - 1);
			((_VLCB *) c)->status = ((_VLCB *) c)->status &
				(~_VCS_IALT);
			c = (Channel *) &(((_VLCB *) c)->in_ws);
			}
		;
		#pragma asm
		LDL	[c]
		.LDC	1
		.LDC	0
		DISC			;Disable channel
		CJ	@L5
		LDL	[tmp]		;Save selected alternative
		STL	[idx]
L5
		#pragma endasm
		tmp++;
		if((((int) *next++) & 1) && (*c != NOPROCESS))
			{
			if(*c != NULL)
				PRun(*c);
			*c = NOPROCESS;
			}
		}
	;
	#pragma asm
	ALTEND
	#pragma endasm

	if(old_priority)
		ProcToLow();

	return (idx);
	}

int
VProcSkipAlt(Channel *c1, ...)		/* c2, c3, ..., cN, 0 */
	{
	VProcSkipAltList(&c1);
	}

int
VProcSkipAltList(Channel **clist)
	{
/*
 *	Note that "reserve" MUST be listed first so that it is assigned
 *	workspace offset zero.  The ALTWT will clobber this location...
 */
	int			reserve;	/* Offset 0 */

	Channel			*c;
	int			idx;
	Channel			**next;
	int			old_priority;
	int			tmp;

	old_priority = ProcToHigh();	/* Change to high priority */

	#pragma asm
	ALT
	#pragma endasm
	next = clist;
	while(*next != 0)
		{
		if(((int) (c = *next++)) & 1)
			{
			c = (Channel *) (((char *) c) - 1);
			if(((_VLCB *) c)->status & _VCS_ORDY)
				((_VLCB *) c)->in_ws = NULL;
			else
				{
				((_VLCB *) c)->status = ((_VLCB *) c)->status |
					_VCS_IALT;
				}
			c = (Channel *) &(((_VLCB *) c)->in_ws);
			}
		;
		#pragma asm
		LDL	[c]
		.LDC	1
		ENBC			;Enable channel
		#pragma endasm
		}
	;
	#pragma asm
	.LDC	1
	ENBS				;Enable skip
	ALTWT
	#pragma endasm
	tmp = 0;
	next = clist;
	while(*next != 0)
		{
		if(((int) (c = *next)) & 1)
			{
			c = (Channel *) (((char *) c) - 1);
			((_VLCB *) c)->status = ((_VLCB *) c)->status &
				(~_VCS_IALT);
			c = (Channel *) &(((_VLCB *) c)->in_ws);
			}
		;
		#pragma asm
		LDL	[c]
		.LDC	1
		.LDC	0
		DISC			;Disable channel
		CJ	@L6
		LDL	[tmp]		;Save selected alternative
		STL	[idx]
L6
		#pragma endasm
		tmp++;
		if((((int) *next++) & 1) && (*c != NOPROCESS))
			{
			if(*c != NULL)
				PRun(*c);
			*c = NOPROCESS;
			}
		}
	;
	#pragma asm
	.LDC	1
	.LDC	0
	DISS				;Disable skip
	CJ	@L7
	.LDC	-1			;Save -1
	STL	[idx]
L7
	ALTEND
	#pragma endasm

	if(old_priority)
		ProcToLow();

	return (idx);
	}

int
VProcTimerAlt(int time,Channel *c1, ...)	/* c2, c3, ..., cN, 0 */
	{
	VProcTimerAltList(time,&c1);
	}

int
VProcTimerAltList(int time,Channel **clist)
	{
/*
 *	Note that "reserve" MUST be listed first so that it is assigned
 *	workspace offset zero.  The TALTWT will clobber this location...
 */
	int			reserve;	/* Offset 0 */

	Channel			*c;
	int			idx;
	Channel			**next;
	int			old_priority;
	int			tmp;
/*
 *	Make initial pass through channel list while at high priority to
 *	perform critical section operations on VLCB's.
 */
	next = clist;
	old_priority = ProcToHigh();	/* Change to high priority */
	while(*next != 0)
		{
		if(((int) (c = *next++)) & 1)
			{
			c = (Channel *) (((char *) c) - 1);
			if(((_VLCB *) c)->status & _VCS_ORDY)
				((_VLCB *) c)->in_ws = NULL;
			else
				{
				((_VLCB *) c)->status = ((_VLCB *) c)->status |
					_VCS_IALT;
				}
			}
		}
	if(old_priority)		/* Restore old priority */
		ProcToLow();
/*
 *	Go through and do the regular TALT stuff.
 */
	;
	#pragma asm
	TALT
	#pragma endasm
	next = clist;
	while(*next != 0)
		{
		if(((int) (c = *next++)) & 1)
			{
			c = (Channel *) &(((_VLCB *) (((char *) c) - 1))->
				in_ws);
			}
		;
		#pragma asm
		LDL	[c]
		.LDC	1
		ENBC			;Enable channel
		#pragma endasm
		}
	;
	#pragma asm
	LDL	[time]
	.LDC	1
	ENBT				;Enable timer
	TALTWT
	#pragma endasm
	tmp = 0;
	next = clist;
	while(*next != 0)
		{
		if(((int) (c = *next++)) & 1)
			{
			c = (Channel *) &(((_VLCB *) (((char *) c) - 1))->
				in_ws);
			}
		;
		#pragma asm
		LDL	[c]
		.LDC	1
		.LDC	0
		DISC			;Disable channel
		CJ	@L8
		LDL	[tmp]		;Save selected alternative
		STL	[idx]
L8
		#pragma endasm
		tmp++;
		}
	;
	#pragma asm
	LDL	[time]
	.LDC	1
	.LDC	0
	DIST				;Disable timer
	CJ	@L9
	.LDC	-1			;Save -1
	STL	[idx]
L9
	ALTEND
	#pragma endasm
/*
 *	Make final pass through channel list while at high priority to
 *	perform critical section operations on VLCB's.
 */
	next = clist;
	old_priority = ProcToHigh();	/* Change to high priority */
	while(*next != 0)
		{
		if(((int) (c = *next++)) & 1)
			{
			c = (Channel *) (((char *) c) - 1);
			((_VLCB *) c)->status = ((_VLCB *) c)->status &
				(~_VCS_IALT);
			c = (Channel *) &(((_VLCB *) c)->in_ws);
			if(*c != NOPROCESS)
				{
				if(*c != NULL)
					PRun(*c);
				*c = NOPROCESS;
				}
			}
		}
	if(old_priority)		/* Restore old priority */
		ProcToLow();

	return (idx);
	}
