/************************************************************************/
/*									*/
/*	File:	vchanio.c						*/
/*									*/
/*	Virtual channel communication non-kernel routines.		*/
/*									*/
/*	The code in this file uses the virtual channel simulation	*/
/*	kernel routines from the "vchan.c" file to implement the rest	*/
/*	of the required interface routines.				*/
/*									*/
/*	Copyright (c) 1991 by Logical Systems.				*/
/*									*/
/*	Written by Kirk Bailey of Logical Systems.			*/
/*									*/
/*	Last update:	8/7/91						*/
/*									*/
/************************************************************************/

#include	<inline.h>
#include	<conc.h>
#include	<cio.h>

#pragma	asm
	.ext	_VChanOut
#pragma	endasm

extern	int	__VChanIn(_VLCB *,void *,int);

/*
 *	VChanInChar
 *
 *	Read and return a single character from the specified virtual channel.
 *
 *	Returns:
 *		The character read.
 */
char
VChanInChar(Channel *chan)
	{
	char			c;

	VChanIn(chan,&c,1);
	return (c);
	}

/*
 *	VChanInInt
 *
 *	Read and return a single integer from the specified virtual channel.
 *
 *	Returns:
 *		The integer read.
 */
int
VChanInInt(Channel *chan)
	{
	int			i;

	VChanIn(chan,&i,(int) (sizeof(i)));
	return (i);
	}

/*
 *	VChanOutChar
 *
 *	Write the specified character out the specified virtual channel.
 *
 *	Returns:
 *		None.
 */
void
VChanOutChar(Channel *chan,int i)
	{
	VChanOut(chan,&i,1);
	}

/*
 *	VChanOutInt
 *
 *	Write the specified integer out the specified virtual channel.
 *
 *	Returns:
 *		None.
 */
void
VChanOutInt(Channel *chan,int i)
	{
	VChanOut(chan,&i,(int) (sizeof(i)));
	}

/*
 *	VChanVIn
 *
 *	Reads no more than the specified number of bytes from the specified
 *	"virtual" channel into memory starting at the specified location.  Note
 *	that this function may NOT be used with regular T-series "soft" or
 *	"hard" channels (ie. the low order bit of the channel address MUST be
 *	a "1").
 *
 *	Returns:
 *		The number of bytes actually read, or -1 if too many/error.
 */
int
VChanVIn(Channel *vchan,void *buf,int size)
	{
	int			old_priority;
	int			rval;
/*
 *	Make sure this function isn't being used with a T-series "hard" or
 *	"soft" channel.
 */
	if((((int) vchan) & 1) != 1)
		PStop();

	if(size < 0)
		return (-1);

	vchan = (Channel *) (((char *) vchan) - 1);	/* Remove bottom bit */

	old_priority = __VChanIn((_VLCB *) vchan,buf,size);	/* Get input */
/*
 *	Compute the return value based on the number of characters actually
 *	received.
 */
	if((rval = ((_VLCB *) vchan)->in_len) > size)
		rval = -1;
/*
 *	If this process was originally at low priority we need to switch back.
 */
	((_VLCB *) vchan)->status &= (~_VCS_ORDY);	/* In case was used */
	((_VLCB *) vchan)->in_ws = NOPROCESS;

	if(old_priority)
		ProcToLow();

	return (rval);
	}

/*
 *	VChanVOut
 *
 *	Writes the specified number of bytes to the specified "virtual"
 *	channel from memory starting at the specified location.  Note that
 *	this function is equivalent to "VChanOut", except is designed for use
 *	in conjunction with "VChanVin", and DOESN'T support regular T-series
 *	"soft" or "hard" channels (the lower order bit of the channel pointer
 *	MUST be non-zero)..
 *
 *	Returns:
 *		None.
 */
void
VChanVOut(Channel *chan,void *buf,int size)
	{
	#pragma	asm
	ldl	[chan]
	ldc	1
	and				;See if special processing needed
	cj	@VCVo1
	j	@_VChanOut
;
;	Either a T-series "soft" channel or a hardware link.  Kill the process
;	as we can't handle these with this function!
;
VCVo1
	stopp
	#pragma	endasm
	}
