/*
 *	vnsstart.c
 *
 *	Handle initialization prior to calling "main".  This file plays the
 *	same role to "vstart.c" as the "nsstart.c" file does to "start.c".
 *	This version handles virtual channels but assumes that the CIO server
 *	will not be used and therefore doesn't request command line argument
 *	information.  A entrypoint is provided to the equivalent of "exit"
 *	under these conditions (ie. "_ns_exit" which is just a "stopp").
 *
 *	Copyright 1988-1991 by Logical Systems.
 *
 *	WARNING:  THE "_ns_vcmain" ROUTINE NEEDS MODIFICATION FOR USE WITH THE
 *	"static link" GLOBAL DATA MODEL (IT MUST SETUP THE INITIAL VALUE)!
 */

#include	<inline.h>

#pragma	asm
	.ext	main			;Main program entry point
	.ext	_boot_chan_in		;Input boot channel address
	.ext	_boot_chan_out		;Output boot channel address
	.ext	ChanIn			;Read input bytes from channel
	.ext	ChanOut			;Write output bytes to channel
	.ext	_host_chan_in		;Host services input channel
	.ext	_host_chan_out		;Host services outptu channel
	.ext	_HostIn			;Read bytes from host services channel
	.ext	_HostOut		;Write bytes to host services channel
	.ext	_node_number		;User supplied network node #
	.ext	_ns_exit		;Replacement for "exit"
	.ext	_VChanInit		;Virtual Channel init function
#pragma	endasm

/*
 *	_ns_vcmain
 *
 *	Non-Server version of "_vcmain".  MUST BE MODIFIED FOR USE WITH THE
 *	STATIC LINK DATA MODEL (THE INITIAL VALUE OF THE STATIC LINK MUST BE
 *	SET AND THE DATA ACCESSES MUST BE CHANGED FROM ".ldc" TO THE
 *	APPROPRIATE TECHNIQUE).
 */
void
_ns_vcmain(void)
	{
	#pragma	asm
	ajw	-2			;A bit of temp space
;
;	At this point WS(1) is the user supplied node #.  If it is set to zero
;	then LD-NET has presumably provided virtual channel configuration info
;	located starting at the address contained in WS(0).  This information
;	is used by the "_VChanInit" function to start up the virtual channel
;	package.
;
;	Input channel address is initially in "C"
;
	or				;Pop channel address into "B"
	rev				;Pop channel address into "A"
	.ldc	_boot_chan_in
	stnl	0			;Save input boot channel address
	.ldc	_boot_chan_in
	ldnl	0
	.ldc	_host_chan_in
	stnl	0			;Dup into host input channel address
	.ldc	ChanIn			;Address of "ChanIn" function
	.ldc	_HostIn			;Dup into "HostIn" function pointer
	stnl	0
	.ldc	4
	bcnt				;Areg will be 8 for T2, 16 for T4/T8
	.ldc	_boot_chan_in
	ldnl	0			;Get input channel address
	xor				;Compute output channel address
	.ldc	_boot_chan_out
	stnl	0			;Save output boot channel address
	.ldc	_boot_chan_out
	ldnl	0
	.ldc	_host_chan_out
	stnl	0			;Dup into host output channel address
	.ldc	ChanOut			;Address of "ChanOut" function
	.ldc	_HostOut		;Dup into "HostOut" function pointer
	stnl	0
;
;	See if we are using virtual channels, and if so, initialize them.
;	If WS(1) is zero then we will be using them...
;
	ldl	1			;See if we need to initialize vchan's
	cj	@main1			;Jump if so
	ldl	1			;Processor node #
	.ldc	_node_number
	stnl	0			;Save processor node #
	j	@main2
main1
	.ldc	0			;Tell _VChanInit to NOT init CIO
	ldl	0			;VChan init data address
	gajw				;Swap into temp workspace
	call	@_VChanInit		;Initialize VChan's
	gajw				;Swap back into original workspace
;
;	Dummy up the argv/argc stuff.
;
main2
	ldlp	1
	stl	0			;argv[0] value
	.ldc	0
	stl	1			;argv[0] points to this
	.ldc	0			;Environment pointer
	ldlp	0			;ARGV
	.ldc	1			;ARGC
	call	@main
	call	_ns_exit		;Kill process, all done
	#pragma	endasm
	}
