/*
 *	vstart.c
 *
 *	Handle initialization prior to calling "main".  This startup code
 *	also supports virtual channels (unlike the otherwise similar
 *	"start.c" file).
 *
 *	Copyright 1987-1991 by Logical Systems.
 *
 *	WARNING:  THE "_vcmain" ROUTINE NEEDS MODIFICATION FOR USE WITH THE
 *	"static link" GLOBAL DATA MODEL (IT MUST SETUP THE INITIAL VALUE)!
 */

#pragma	asm
	.ext	main			;Main program entry point
	.ext	_arg_get		;Get argv table and args
	.ext	_arg_size		;Get size of argv table and args
	.ext	_boot_chan_in		;Input boot channel address
	.ext	_boot_chan_out		;Output boot channel address
	.ext	ChanIn			;Read input bytes from channel
	.ext	ChanOut			;Write output bytes to channel
	.ext	exit			;Exit routine
	.ext	_host_chan_in		;Host services input channel
	.ext	_host_chan_out		;Host services output channel
	.ext	_HostIn			;Read bytes from host services channel
	.ext	_HostOut		;Write bytes to host services channel
	.ext	_node_number		;User supplied network node #
	.ext	_VChanInit		;Virtual Channel init function
#pragma	endasm

/*
 *	_vcmain
 *
 *	MUST BE MODIFIED FOR USE WITH THE STATIC LINK DATA MODEL (THE INITIAL
 *	VALUE OF THE STATIC LINK MUST BE SET AND THE DATA ACCESSES MUST BE
 *	CHANGED FROM ".ldc" TO THE APPROPRIATE TECHNIQUE).
 */
_vcmain()
	{
#pragma	asm
	ajw	-2			;A bit of temp space
;
;	At this point WS(1) is the user supplied node #.  If it is set to zero
;	then LD-NET has presumably provided virtual channel configuration info
;	located starting at the address contained in WS(0).  This information
;	is used by the "_VChanInit" function to start up the virtual channel
;	package.
;
;	The parent input channel address is initially in "C"
;
	or				;Pop channel address into "B"
	rev				;Pop channel address into "A"
	.ldc	_boot_chan_in
	stnl	0			;Save input boot channel address
	.ldc	4
	bcnt				;Areg will be 8 for T2, 16 for T4/T8
	.ldc	_boot_chan_in
	ldnl	0			;Get input boot channel address
	xor				;Compute output channel address
	.ldc	_boot_chan_out
	stnl	0			;Save output boot channel address
	ldl	1			;See if we need to initialize VChan's
	eqc	0
	cj	@main1			;Jump if not
;
;	Using Virtual Channels, initialize them.  Also pass a non-zero
;	second parameter to "_VChanInit" so that it is allowed to initialize
;	the host server (CIO), as to maximum packet sizes, etc.
;
	.ldc	1			;Tell _VChanInit to init CIO
	ldl	0			;Get VChan init data address
	gajw				;Swap into the new workspace
	call	@_VChanInit		;Initialize VChan's
	gajw				;Swap back to old workspace
	j	@main2
main1
;
;	Not using Virtual Channels, initialize in the normal fashion.
;
	.ldc	_boot_chan_in		;Get input boot channel address
	ldnl	0
	.ldc	_host_chan_in
	stnl	0			;Dup into host input channel address
	.ldc	ChanIn			;Address of "ChanIn" function
	.ldc	_HostIn			;Dup into "HostIn" function pointer
	stnl	0
	.ldc	_boot_chan_out		;Get output boot channel address
	ldnl	0
	.ldc	_host_chan_out
	stnl	0			;Dup into host output channel address
	.ldc	ChanOut			;Address of "ChanOut" function
	.ldc	_HostOut		;Dup into "HostOut" function pointer
	stnl	0
	ldl	1			;Processor node #
	.ldc	_node_number
	stnl	0			;Save processor node #
	ldl	1
	eqc	1			;If node #1, get argv/argc stuff
	cj	@main3			;Jump if not
main2
;
;	Whether we are using virtual channels, or are the root node, get the
;	"argc"/"argv" stuff from CIO.
;
	call	@_arg_size		;Get needed size of argv table and args
	.ldc	2
	bcnt
	diff				;Since we already have two words...
	ldlp	0			;Point to current WS
	rev
	diff				;Compute new WS
	gajw			
	ldlp	0			;Current WS
	call	@_arg_get		;Get argv vector and arguments
	.ldc	0
	rev				;B is NULL env ptr, A is argc
	ldlp	0
	rev				;C is env ptr, B is argv, A is argc
	call	@main			;Call user program
	j	@main4			;Default to non error exit
main3
;
;	We are not using Virtual Channels and we aren't the root node, dummy up
;	the "argc"/"argv" stuff.
;
	ldlp	1
	stl	0			;argv[0] value
	.ldc	0
	stl	1			;argv[0] points to this
	.ldc	0			;Environment pointer
	ldlp	0			;ARGV
	.ldc	1			;ARGC
	call	@main
main4
	.ldc	0
	call	@exit			;Default to non error exit
#pragma	endasm
	}
