/************************************************************************/
/*									*/
/*	File:	tio.c							*/
/*									*/
/*	Primitive Transputer I/O server for the INMOS Transputer.	*/
/*	This is a simple prototype for the actual "C" I/O server	*/
/*	("cio.c"), but may be independently useful as a basis for	*/
/*	developing other servers for special applications.		*/
/*									*/
/*	This program must be linked with a version of the INMOS		*/
/*	"link.c" I/O interface to customize it to the associated	*/
/*	hardware!							*/
/*									*/
/*		Written by Kirk Bailey of Logical Systems.		*/
/*									*/
/*		Copyright (c) 1986-1988 by Logical Systems.		*/
/*									*/
/*				11/2/88					*/
/*									*/
/************************************************************************/

#include	<stdio.h>		/* Standard I/O include file */
#include	<stdlib.h>		/* Standard library include file */
#include	<conio.h>		/* Console I/O stuff */
#include	"taldef.h"		/* Assembler/Linker definitions */
#include	"tload.h"		/* Transputer downloading info */

/************************************************************************/
/*									*/
/*			Global Data Definitions				*/
/*									*/
/************************************************************************/

int	LinkId		=	0;	/* LinkId from "OpenLink" */

#ifdef	__TURBOC__
extern	unsigned	_stklen = 8192;
#endif	/* __TURBOC__ */

/************************************************************************/
/*									*/
/*			Function Type Declarations			*/
/*									*/
/************************************************************************/

VOID	banner();
VOID	fatal();
int	main();

/*
 *	The following functions may be called only for side effects.
 */
/*lint +fvr */

/*lint -fvr */

/************************************************************************/
/*									*/
/*	Routine:	main						*/
/*									*/
/*	Opens up communication with the transputer link adapter on a	*/
/*	INMOS Transputer and funnels standard input & output in and out	*/
/*	of the connected transputer.					*/
/*									*/
/*	Parameters:							*/
/*		"argc"	An integer count of the command line arguments.	*/
/*		"argv"	A pointer to an array of pointers which point	*/
/*			to individual command line arguments.		*/
/*									*/
/*	Returns:	None.						*/
/*									*/
/************************************************************************/

#ifdef	__TURBOC__
#pragma	warn	-par
#endif	/* __TURBOC__ */

int
main(argc,argv)
	int			argc;
	char			*argv[];
	{
	char			buf[2];
	STRING			open_str;

	banner();
	open_str = getenv(LINK_NAME);	/* Get desired link name if present */
	LinkId = OpenLink(open_str);	/* Open specified link channel */
	if(open_str == NULL)
		open_str = "<default>";
	if(LinkId <= 0)
		fatal("Unable to open link: ",open_str);
/*
 *	Echo the transputer links to standard I/O.  Note that "kbhit" is a DOS
 *	specific way of determining if a character is available on standard
 *	input...
 */
	buf[1] = '\0';		/* Only dealing with a byte at a time */
	FOREVER
		{
		if(kbhit() && TestWrite(LinkId))
			{
			if(TestWrite(LinkId) < 0)
				{
				fatal("Error testing write status on link: ",
					open_str);
				}
			buf[0] = getch();
			if(WriteLink(LinkId,buf,1,INFINITE_TIMEOUT) < 0)
				fatal("Error writing to link: ",open_str);
			}
		if(TestRead(LinkId))
			{
			if(TestRead(LinkId) < 0)
				{
				fatal("Error testing read status on link: ",
					open_str);
				}
			if(ReadLink(LinkId,buf,1,INFINITE_TIMEOUT) < 0)
				fatal("Error reading from link: ",open_str);
			putchar(buf[0]);
			}
		}
/*
 *	Note that the above loop must be modified with whatever signal
 *	catching code is needed so that the following code gets executed if
 *	the execution of TIO is aborted on a REAL computer (one which cares
 *	about matching link "open" and "close" operations...)
 */
#ifdef	REAL_COMPUTER
	if(CloseLink(LinkId) < 0)
		{
		LinkId = 0;
		fatal("Unable to close link: ",open_str);
		}
	exit(NOERRORS);			/* Head for the barn */
#endif	/* REAL_COMPUTER */
	}

#ifdef	__TURBOC__
#pragma	warn	.par
#endif	/* __TURBOC__ */

/************************************************************************/
/*									*/
/*	Routine:	banner						*/
/*									*/
/*	Display sign-on message to user.				*/
/*									*/
/*	Parameters:	None.						*/
/*									*/
/*	Returns:	None.						*/
/*									*/
/************************************************************************/

VOID
banner()
	{
	fprintf(stderr,
		"TIO (Transparent I/O Server), %s [Link I/O Driver: '%s']\n",
			VERSION,IODRIVER);
	fprintf(stderr,"%s\n\n",COPYRIGHT);
	}

/************************************************************************/
/*									*/
/*	Routine:	fatal						*/
/*									*/
/*	Display error message to user and play dead.			*/
/*									*/
/*	Parameters:							*/
/*		"str1"	The first part of the error message to display.	*/
/*		"str2"	The second part of the error message.		*/
/*									*/
/*	Returns:	None.						*/
/*									*/
/************************************************************************/

VOID
fatal(str1,str2)
	STRING			str1;
	STRING			str2;
	{
	fprintf(stderr,"FATAL: %s%s\n",str1,str2);
	if(LinkId > 0)
		(VOID) CloseLink(LinkId);
	exit(ERRORS);
	}
