# =============================================================================
# Skeleton make file for Logical Systems C programs.
# D. Thiebaut
#
# To be used with Logical Systems' MAKE utility
#
# To use, fill in first five lines after header with the information requested
# as in this "before" and "after" example.
#
#  Before
#  ======================
#  FILENAME    = [Enter here name of main program (no .c extension)]
#  FILE_NAMES  = $(FILENAME) [name (no extension) of other .c files needed]
#  C_FILES     = $(FILENAME).c [same as above, but this time with .c extension]
#  LINK_FILES  = $(FILENAME).trl $(FILENAME).lnk [same as above, with .trl ext]
#  HEADERS     = [any header files used to generate main program]
#
#  After
#  ======================
#  FILENAME    = queenslb
#  FILE_NAMES  = $(FILENAME) balance heap search
#  C_FILES     = $(FILENAME).c balance.c heap.c search.c
#  LINK_FILES  = $(FILENAME).trl $(FILENAME).lnk balance.trl heap.trl search.trl
#  HEADERS     = balance.h heap.h search.h
# =============================================================================

# = MACROS ====================================================================
FILENAME    = [Enter here name of main program (no .c extension)]
FILE_NAMES  = $(FILENAME) [name (no extension) of other .c files needed]
C_FILES     = $(FILENAME).c [same as above, but this time with .c extension]
LINK_FILES  = $(FILENAME).trl $(FILENAME).lnk [same as above, with .trl ext]
HEADERS     = [any header files used to generate main program]

# = FLAGS USED BY PREPROCESSOR, COMPILER, AND LINKER ==========================
PP_FLAGS    = -dT805 -v
TCX_FLAGS   = -p85 -v
TASM_FLAGS  = -t -v

# = DEPENDENCIES ==============================================================
$(FILENAME).tld:   $(LINK_FILES)

$(FILENAME).trl:   $(FILENAME).tal

$(FILENAME).tal:   $(FILENAME).pp

# = ADD OTHER DEPENDENCIES HERE IF NECESSARY ===============================
# example:
# balance.pp:        balance.c balance.h

$(FILENAME).pp:    $(FILENAME).c $(HEADERS)

$(FILENAME).lnk:    $(FILENAME).mak
      ECHO FLAG                          > $(FILENAME).lnk
      ECHO LIST     $(FILENAME).map      >> $(FILENAME).lnk
      ECHO INPUT    $(FILE_NAMES)        >> $(FILENAME).lnk
      ECHO ENTRY    _vcmain              >> $(FILENAME).lnk
      ECHO LIB      t8lib                >> $(FILENAME).lnk
      ECHO LOAD     0x80002000           >> $(FILENAME).lnk
      ECHO STACK    0x80001F00           >> $(FILENAME).lnk

# = IMPLIED RULES ==========================================================
.c.pp:
      pp $< $(PP_FLAGS)

.pp.tal:
      tcx $< $(TCX_FLAGS)

.tal.trl:
      ttasm $< $(TASM_FLAGS)

.trl.tld:
      tlnk $(FILENAME).lnk

